@@ -11,6 +11,9 @@ import aiosqlite
 import freezegun
 import pytest
 
+from tests.async_mock import AsyncMock, MagicMock, call, patch
+from tests.conftest import make_app, make_ieee, make_node_desc
+from tests.test_backups import backup_factory  # noqa: F401
 from zigpy import profiles
 import zigpy.appdb
 import zigpy.application
@@ -22,17 +25,14 @@ import zigpy.ota
 from zigpy.quirks import CustomDevice
 import zigpy.types as t
 import zigpy.zcl
+from zigpy.zcl.clusters.general import Basic
 from zigpy.zcl.foundation import Status as ZCLStatus
 from zigpy.zdo import types as zdo_t
 
-from tests.async_mock import AsyncMock, MagicMock, call, patch
-from tests.conftest import make_app, make_ieee
-from tests.test_backups import backup_factory  # noqa: F401
-
 
 @pytest.fixture(autouse=True)
 def auto_kill_aiosqlite():
-    """aiosqlite's background thread does not let pytest exit when a failure occurs"""
+    """Aiosqlite's background thread does not let pytest exit when a failure occurs"""
     yield
 
     for thread in threading.enumerate():
@@ -136,18 +136,22 @@ async def test_database(tmp_path):
     ep.status = zigpy.endpoint.Status.ZDO_INIT
     ep.profile_id = 260
     ep.device_type = 0xFFFD  # Invalid
-    clus = ep.add_input_cluster(0)
-    ep.add_output_cluster(1)
+    in_clus = ep.add_input_cluster(0)
+    out_clus = ep.add_output_cluster(0)
     ep = dev.add_endpoint(3)
     ep.status = zigpy.endpoint.Status.ZDO_INIT
     ep.profile_id = 49246
     ep.device_type = profiles.zll.DeviceType.COLOR_LIGHT
     app.device_initialized(dev)
-    clus.update_attribute(0, 99)
-    clus.update_attribute(4, bytes("Custom", "ascii"))
-    clus.update_attribute(5, bytes("Model", "ascii"))
-    clus.listener_event("cluster_command", 0)
-    clus.listener_event("general_command")
+
+    in_clus.update_attribute(0, 99)
+    in_clus.update_attribute(4, bytes("Custom", "ascii"))
+    in_clus.update_attribute(5, bytes("Model", "ascii"))
+    in_clus.listener_event("cluster_command", 0)
+    in_clus.listener_event("general_command")
+
+    out_clus.update_attribute(0, 99)
+
     dev.relays = relays_1
     signature = dev.get_signature()
     assert ep.endpoint_id in signature[SIG_ENDPOINTS]
@@ -198,9 +202,10 @@ async def test_database(tmp_path):
     assert dev.endpoints[2].in_clusters[0]._attr_cache[0] == 99
     assert dev.endpoints[2].in_clusters[0]._attr_cache[4] == bytes("Custom", "ascii")
     assert dev.endpoints[2].in_clusters[0]._attr_cache[5] == bytes("Model", "ascii")
+    assert dev.endpoints[2].out_clusters[0].cluster_id == 0x0000
+    assert dev.endpoints[2].out_clusters[0]._attr_cache[0] == 99
     assert dev.endpoints[2].manufacturer == "Custom"
     assert dev.endpoints[2].model == "Model"
-    assert dev.endpoints[2].out_clusters[1].cluster_id == 1
     assert dev.endpoints[3].device_type == profiles.zll.DeviceType.COLOR_LIGHT
     assert dev.relays == relays_1
     # The timestamp won't be restored exactly but it is more than close enough
@@ -411,7 +416,7 @@ async def test_groups(mock_request, tmp_path):
     await app5.shutdown()
 
 
-@pytest.mark.parametrize("dev_init", (True, False))
+@pytest.mark.parametrize("dev_init", [True, False])
 async def test_attribute_update(tmp_path, dev_init):
     """Test attribute update for initialized and uninitialized devices."""
 
@@ -485,7 +490,7 @@ async def test_attribute_update_short_interval(tmp_path):
     attr_update_time_first = clus._attr_last_updated[0x4000]
 
     # update attribute again 10 seconds later
-    fake_time = datetime.utcnow() + timedelta(seconds=10)
+    fake_time = datetime.now(timezone.utc) + timedelta(seconds=10)
     with freezegun.freeze_time(fake_time):
         clus.update_attribute(0x4000, "2.0")
 
@@ -710,7 +715,7 @@ async def test_stopped_appdb_listener(tmp_path):
 
 @patch.object(Device, "schedule_initialize", new=mock_dev_init(True))
 async def test_invalid_node_desc(tmp_path):
-    """devices without a valid node descriptor should not save the node descriptor."""
+    """Devices without a valid node descriptor should not save the node descriptor."""
 
     ieee_1 = make_ieee(1)
     nwk_1 = 0x1111
@@ -767,7 +772,7 @@ async def test_appdb_worker_exception(tmp_path):
     assert save_mock.await_count == 3
 
 
-@pytest.mark.parametrize("dev_init", (True, False))
+@pytest.mark.parametrize("dev_init", [True, False])
 async def test_unsupported_attribute(tmp_path, dev_init):
     """Test adding unsupported attributes for initialized and uninitialized devices."""
 
@@ -785,13 +790,16 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     ep.status = zigpy.endpoint.Status.ZDO_INIT
     ep.profile_id = 260
     ep.device_type = profiles.zha.DeviceType.PUMP
-    clus = ep.add_input_cluster(0)
-    ep.add_output_cluster(1)
-    clus.update_attribute(4, "Custom")
-    clus.update_attribute(5, "Model")
+    in_clus = ep.add_input_cluster(0)
+    in_clus.update_attribute(4, "Custom")
+    in_clus.update_attribute(5, "Model")
     app.device_initialized(dev)
-    clus.add_unsupported_attribute(0x0010)
-    clus.add_unsupported_attribute("physical_env")
+
+    in_clus.add_unsupported_attribute(0x0010)
+    in_clus.add_unsupported_attribute("physical_env")
+
+    out_clus = ep.add_output_cluster(0)
+    out_clus.add_unsupported_attribute(0x0010)
     await app.shutdown()
 
     # Everything should've been saved - check that it re-loads
@@ -800,7 +808,9 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
     assert 0x0010 in dev.endpoints[3].in_clusters[0].unsupported_attributes
+    assert 0x0010 in dev.endpoints[3].out_clusters[0].unsupported_attributes
     assert "location_desc" in dev.endpoints[3].in_clusters[0].unsupported_attributes
+    assert "location_desc" in dev.endpoints[3].out_clusters[0].unsupported_attributes
     assert 0x0011 in dev.endpoints[3].in_clusters[0].unsupported_attributes
     assert "physical_env" in dev.endpoints[3].in_clusters[0].unsupported_attributes
     await app2.shutdown()
@@ -818,15 +828,19 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     dev = app3.get_device(ieee)
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
-    cluster = dev.endpoints[3].in_clusters[0]
-    assert 0x0010 in dev.endpoints[3].in_clusters[0].unsupported_attributes
-    cluster.request = mockrequest
-    await cluster.read_attributes([0x0010], allow_cache=False)
-    assert 0x0010 not in dev.endpoints[3].in_clusters[0].unsupported_attributes
-    assert "location_desc" not in dev.endpoints[3].in_clusters[0].unsupported_attributes
-    assert dev.endpoints[3].in_clusters[0].get(0x0010) == "Not Removed"
-    assert 0x0011 in dev.endpoints[3].in_clusters[0].unsupported_attributes
-    assert "physical_env" in dev.endpoints[3].in_clusters[0].unsupported_attributes
+
+    in_cluster = dev.endpoints[3].in_clusters[0]
+    assert 0x0010 in in_cluster.unsupported_attributes
+    in_cluster.request = mockrequest
+    await in_cluster.read_attributes([0x0010], allow_cache=False)
+    assert 0x0010 not in in_cluster.unsupported_attributes
+    assert "location_desc" not in in_cluster.unsupported_attributes
+    assert in_cluster.get(0x0010) == "Not Removed"
+    assert 0x0011 in in_cluster.unsupported_attributes
+    assert "physical_env" in in_cluster.unsupported_attributes
+
+    out_cluster = dev.endpoints[3].out_clusters[0]
+    out_cluster.remove_unsupported_attribute(0x0010)
     await app3.shutdown()
 
     # Everything should've been saved - check that it re-loads
@@ -835,6 +849,7 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
     assert 0x0010 not in dev.endpoints[3].in_clusters[0].unsupported_attributes
+    assert 0x0010 not in dev.endpoints[3].out_clusters[0].unsupported_attributes
     assert dev.endpoints[3].in_clusters[0].get(0x0010) == "Not Removed"
     assert "location_desc" not in dev.endpoints[3].in_clusters[0].unsupported_attributes
     assert 0x0011 in dev.endpoints[3].in_clusters[0].unsupported_attributes
@@ -968,7 +983,7 @@ async def test_last_seen(tmp_path):
     # Last-seen is only written to the db every 30s (no write case)
     now = datetime.fromtimestamp(dev.last_seen + 5, timezone.utc)
     with freezegun.freeze_time(now):
-        dev.update_last_seen()
+        dev.last_seen = datetime.now(timezone.utc)
 
     await app.shutdown()
 
@@ -983,7 +998,7 @@ async def test_last_seen(tmp_path):
     # Last-seen is only written to the db every 30s (write case)
     now = datetime.fromtimestamp(dev.last_seen + 35, timezone.utc)
     with freezegun.freeze_time(now):
-        dev.update_last_seen()
+        dev.last_seen = datetime.now(timezone.utc)
 
     await app.shutdown()
 
@@ -995,7 +1010,7 @@ async def test_last_seen(tmp_path):
 
 
 @pytest.mark.parametrize(
-    "stdlib_version,use_sqlite",
+    ("stdlib_version", "use_sqlite"),
     [
         ((1, 0, 0), False),
         ((2, 0, 0), False),
@@ -1009,8 +1024,9 @@ def test_pysqlite_load_success(stdlib_version, use_sqlite):
     pysqlite3 = MagicMock()
     pysqlite3.sqlite_version_info = (3, 30, 0)
 
-    with patch.dict(sys.modules, {"pysqlite3": pysqlite3}), patch.object(
-        sys.modules["sqlite3"], "sqlite_version_info", new=stdlib_version
+    with (
+        patch.dict(sys.modules, {"pysqlite3": pysqlite3}),
+        patch.object(sys.modules["sqlite3"], "sqlite_version_info", new=stdlib_version),
     ):
         module = zigpy.appdb._import_compatible_sqlite3(zigpy.appdb.MIN_SQLITE_VERSION)
 
@@ -1021,15 +1037,14 @@ def test_pysqlite_load_success(stdlib_version, use_sqlite):
 
 
 @pytest.mark.parametrize(
-    "stdlib_version,pysqlite3_version",
+    ("stdlib_version", "pysqlite3_version"),
     [
         ((1, 0, 0), None),
         ((1, 0, 0), (1, 0, 1)),
     ],
 )
 def test_pysqlite_load_failure(stdlib_version, pysqlite3_version):
-    """
-    Test that the internal import SQLite helper will throw an error when no compatible
+    """Test that the internal import SQLite helper will throw an error when no compatible
     module can be found.
     """
 
@@ -1040,8 +1055,9 @@ def test_pysqlite_load_failure(stdlib_version, pysqlite3_version):
     else:
         pysqlite3_patch = patch.dict(sys.modules, {"pysqlite3": None})
 
-    with pysqlite3_patch, patch.object(
-        sys.modules["sqlite3"], "sqlite_version_info", new=stdlib_version
+    with (
+        pysqlite3_patch,
+        patch.object(sys.modules["sqlite3"], "sqlite_version_info", new=stdlib_version),
     ):
         with pytest.raises(RuntimeError):
             zigpy.appdb._import_compatible_sqlite3(zigpy.appdb.MIN_SQLITE_VERSION)
@@ -1075,9 +1091,7 @@ async def test_appdb_network_backups(tmp_path, backup_factory):  # noqa: F811
     await app3.shutdown()
 
 
-async def test_appdb_network_backups_format_change(
-    tmp_path, backup_factory
-):  # noqa: F811
+async def test_appdb_network_backups_format_change(tmp_path, backup_factory):  # noqa: F811
     db = tmp_path / "test.db"
 
     backup = backup_factory()
@@ -1118,3 +1132,46 @@ async def test_appdb_persist_coordinator_info(tmp_path):  # noqa: F811
         await app.shutdown()
 
     assert mock_save_attr_cache.mock_calls == [call(app._device.endpoints[1])]
+
+
+async def test_appdb_attribute_clear(tmp_path):
+    db = tmp_path / "test.db"
+    app = await make_app_with_db(db)
+
+    dev = app.add_device(nwk=0x1234, ieee=t.EUI64.convert("aa:bb:cc:dd:11:22:33:44"))
+    dev.node_desc = make_node_desc(logical_type=zdo_t.LogicalType.Router)
+
+    ep = dev.add_endpoint(1)
+    ep.status = zigpy.endpoint.Status.ZDO_INIT
+    ep.profile_id = 260
+    ep.device_type = profiles.zha.DeviceType.PUMP
+
+    basic = ep.add_input_cluster(Basic.cluster_id)
+    app.device_initialized(dev)
+
+    basic.update_attribute(Basic.AttributeDefs.zcl_version.id, 0x12)
+
+    await app.shutdown()
+
+    # Upon reload, the attribute exists and is in the cache
+    app2 = await make_app_with_db(db)
+    dev2 = app2.get_device(ieee=dev.ieee)
+    assert (
+        dev2.endpoints[1].basic._attr_cache[Basic.AttributeDefs.zcl_version.id] == 0x12
+    )
+
+    # Clear an existing attribute
+    dev2.endpoints[1].basic.update_attribute(Basic.AttributeDefs.zcl_version.id, None)
+
+    # Clear an attribute not in the cache
+    dev2.endpoints[1].basic.update_attribute(Basic.AttributeDefs.manufacturer.id, None)
+
+    assert Basic.AttributeDefs.zcl_version.id not in dev2.endpoints[1].basic._attr_cache
+    await asyncio.sleep(0.1)
+    await app2.shutdown()
+
+    # The attribute has been removed from the database
+    app3 = await make_app_with_db(db)
+    dev3 = app3.get_device(ieee=dev.ieee)
+    assert Basic.AttributeDefs.zcl_version.id not in dev3.endpoints[1].basic._attr_cache
+    await app3.shutdown()
