@@ -1,3 +1,4 @@
+from datetime import datetime, timezone
 import logging
 import pathlib
 from sqlite3.dump import _iterdump as iterdump
@@ -5,16 +6,15 @@ from sqlite3.dump import _iterdump as iterdump
 from aiosqlite.context import contextmanager
 import pytest
 
+from tests.async_mock import AsyncMock, MagicMock, patch
+from tests.conftest import app  # noqa: F401
+from tests.test_appdb import auto_kill_aiosqlite, make_app_with_db  # noqa: F401
 import zigpy.appdb
 from zigpy.appdb import sqlite3
 import zigpy.appdb_schemas
 import zigpy.types as t
 from zigpy.zdo import types as zdo_t
 
-from tests.async_mock import AsyncMock, MagicMock, patch
-from tests.conftest import app  # noqa: F401
-from tests.test_appdb import auto_kill_aiosqlite, make_app_with_db  # noqa: F401
-
 
 @pytest.fixture
 def test_db(tmp_path):
@@ -265,7 +265,7 @@ async def test_migration_missing_node_descriptor(test_db, caplog):
 
 
 @pytest.mark.parametrize(
-    "fail_on_sql,fail_on_count",
+    ("fail_on_sql", "fail_on_count"),
     [
         ("INSERT INTO node_descriptors_v4 VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)", 0),
         ("INSERT INTO neighbors_v4 VALUES (?,?,?,?,?,?,?,?,?,?,?,?)", 5),
@@ -485,7 +485,7 @@ async def test_last_seen_initial_migration(test_db):
     dev = app.get_device(nwk=0xBD4D)
 
     assert dev.last_seen is None
-    dev.update_last_seen()
+    dev.last_seen = datetime.now(timezone.utc)
     assert isinstance(dev.last_seen, float)
     await app.shutdown()
 
