@@ -1,9 +1,14 @@
 import asyncio
+import importlib.util
 import itertools
+import pathlib
+import pkgutil
+import sys
 from typing import Final
 
 import pytest
 
+from zigpy import zcl
 from zigpy.const import (
     SIG_ENDPOINTS,
     SIG_EP_INPUT,
@@ -20,7 +25,6 @@ import zigpy.endpoint
 import zigpy.quirks
 from zigpy.quirks.registry import DeviceRegistry
 import zigpy.types as t
-import zigpy.zcl as zcl
 
 from .async_mock import AsyncMock, MagicMock, patch, sentinel
 
@@ -172,13 +176,13 @@ def test_model_manuf_device_sig(real_device):
 
 def test_custom_devices():
     def _check_range(cluster):
-        for range in zcl.Cluster._registry_range:
-            if range[0] <= cluster <= range[1]:
+        for left, right in zcl.Cluster._registry_range:
+            if left <= cluster <= right:
                 return True
         return False
 
     # Validate that all CustomDevices look sane
-    reg = zigpy.quirks._DEVICE_REGISTRY.registry
+    reg = zigpy.quirks._DEVICE_REGISTRY.registry_v1
     candidates = list(
         itertools.chain(*itertools.chain(*[m.values() for m in reg.values()]))
     )
@@ -557,11 +561,11 @@ def manuf_cluster2():
 
 
 @pytest.mark.parametrize(
-    "cmd_name, manufacturer",
-    (
+    ("cmd_name", "manufacturer"),
+    [
         ("client_cmd0", None),
         ("client_cmd1", sentinel.manufacturer_id),
-    ),
+    ],
 )
 async def test_client_cmd_vendor_specific_by_name(
     manuf_cluster, manuf_cluster2, cmd_name, manufacturer
@@ -581,11 +585,11 @@ async def test_client_cmd_vendor_specific_by_name(
 
 
 @pytest.mark.parametrize(
-    "cmd_name, manufacturer",
-    (
+    ("cmd_name", "manufacturer"),
+    [
         ("server_cmd0", None),
         ("server_cmd1", sentinel.manufacturer_id),
-    ),
+    ],
 )
 async def test_srv_cmd_vendor_specific_by_name(
     manuf_cluster, manuf_cluster2, cmd_name, manufacturer
@@ -605,11 +609,11 @@ async def test_srv_cmd_vendor_specific_by_name(
 
 
 @pytest.mark.parametrize(
-    "attr_name, manufacturer",
-    (
+    ("attr_name", "manufacturer"),
+    [
         ("attr0", None),
         ("attr1", sentinel.manufacturer_id),
-    ),
+    ],
 )
 async def test_read_attr_manufacture_specific(
     manuf_cluster, manuf_cluster2, attr_name, manufacturer
@@ -639,11 +643,11 @@ async def test_read_attr_manufacture_specific(
 
 
 @pytest.mark.parametrize(
-    "attr_name, manufacturer",
-    (
+    ("attr_name", "manufacturer"),
+    [
         ("attr0", None),
         ("attr1", sentinel.manufacturer_id),
-    ),
+    ],
 )
 async def test_write_attr_manufacture_specific(
     manuf_cluster, manuf_cluster2, attr_name, manufacturer
@@ -673,11 +677,11 @@ async def test_write_attr_manufacture_specific(
 
 
 @pytest.mark.parametrize(
-    "attr_name, manufacturer",
-    (
+    ("attr_name", "manufacturer"),
+    [
         ("attr0", None),
         ("attr1", sentinel.manufacturer_id),
-    ),
+    ],
 )
 async def test_write_attr_undivided_manufacture_specific(
     manuf_cluster, manuf_cluster2, attr_name, manufacturer
@@ -711,11 +715,11 @@ async def test_write_attr_undivided_manufacture_specific(
 
 
 @pytest.mark.parametrize(
-    "attr_name, manufacturer",
-    (
+    ("attr_name", "manufacturer"),
+    [
         ("attr0", None),
         ("attr1", sentinel.manufacturer_id),
-    ),
+    ],
 )
 async def test_configure_reporting_manufacture_specific(
     manuf_cluster, manuf_cluster2, attr_name, manufacturer
@@ -960,7 +964,7 @@ async def test_manuf_id_disable(real_device):
     assert len(request_mock.mock_calls) == 3
 
     for mock_call in request_mock.mock_calls:
-        data = mock_call.args[2]
+        data = mock_call.kwargs["data"]
         hdr, _ = zcl.foundation.ZCLHeader.deserialize(data)
         assert hdr.manufacturer == 0x1234
 
@@ -981,11 +985,33 @@ async def test_manuf_id_disable(real_device):
     assert len(request_mock.mock_calls) == 3
 
     for mock_call in request_mock.mock_calls:
-        data = mock_call.args[2]
+        data = mock_call.kwargs["data"]
         hdr, _ = zcl.foundation.ZCLHeader.deserialize(data)
         assert hdr.manufacturer is None
 
 
+async def test_cluster_manufacturer_id_override(real_device):
+    class TestCluster(ManufacturerSpecificCluster):
+        cluster_id = 0xFF00
+        manufacturer_id_override = 0xABCD
+
+    real_device.manufacturer_id_override = 0x1234
+
+    ep = real_device.endpoints[1]
+    ep.add_input_cluster(TestCluster.cluster_id, TestCluster(ep))
+    assert isinstance(ep.just_a_cluster, TestCluster)
+
+    assert ep.manufacturer_id == 0x1234
+
+    with patch.object(ep, "request", AsyncMock()) as request_mock:
+        await ep.just_a_cluster.read_attributes(["attr0"])
+
+    # We prefer the cluster-level override
+    data = request_mock.mock_calls[0].kwargs["data"]
+    hdr, _ = zcl.foundation.ZCLHeader.deserialize(data)
+    assert hdr.manufacturer == 0xABCD
+
+
 async def test_request_with_kwargs(real_device):
     class CustomLevel(zigpy.quirks.CustomCluster, zcl.clusters.general.LevelControl):
         pass
@@ -1031,3 +1057,116 @@ async def test_request_with_kwargs(real_device):
 
         assert len(request_mock.mock_calls) == 3
         assert all(c == request_mock.mock_calls[0] for c in request_mock.mock_calls)
+
+
+def test_purge_custom_quirks(tmp_path: pathlib.Path, app_mock) -> None:
+    def load_quirks():
+        for importer, modname, _ in pkgutil.walk_packages(path=[str(tmp_path)]):
+            spec = importer.find_spec(modname)
+            module = importlib.util.module_from_spec(spec)
+            sys.modules[modname] = module
+            spec.loader.exec_module(module)
+
+    (tmp_path / "quirk1.py").write_text("""
+import zigpy.quirks
+from zigpy.zcl.clusters.general import LevelControl
+from zigpy.const import (
+    SIG_ENDPOINTS,
+    SIG_EP_INPUT,
+    SIG_EP_OUTPUT,
+    SIG_EP_PROFILE,
+    SIG_EP_TYPE,
+    SIG_MODELS_INFO,
+)
+
+class CustomLevel1(zigpy.quirks.CustomCluster, LevelControl):
+    pass
+
+class TestQuirk1(zigpy.quirks.CustomDevice):
+    signature = {
+        SIG_MODELS_INFO: (("manufacturer1", "model1"),),
+        SIG_ENDPOINTS: {
+            1: {
+                SIG_EP_PROFILE: 255,
+                SIG_EP_TYPE: 255,
+                SIG_EP_INPUT: [3],
+                SIG_EP_OUTPUT: [6],
+            }
+        },
+    }
+
+    replacement = {
+        SIG_ENDPOINTS: {
+            1: {
+                SIG_EP_PROFILE: 255,
+                SIG_EP_TYPE: 255,
+                SIG_EP_INPUT: [3, CustomLevel1],
+                SIG_EP_OUTPUT: [6],
+            }
+        },
+    }""")
+
+    (tmp_path / "quirk2.py").write_text("""
+import zigpy.quirks
+
+from zigpy.quirks.v2 import QuirkBuilder
+from zigpy.zcl import ClusterType
+from zigpy.zcl.clusters.general import LevelControl
+
+class CustomLevel2(zigpy.quirks.CustomCluster, LevelControl):
+    pass
+
+QuirkBuilder("manufacturer2", "model2").adds(
+    cluster=CustomLevel2,
+    cluster_type=ClusterType.Server,
+    endpoint_id=1,
+).add_to_registry()
+""")
+
+    dev1 = zigpy.device.Device(
+        app_mock, t.EUI64.convert("11:11:11:11:11:11:11:11"), 0x1234
+    )
+    dev1.add_endpoint(1)
+    dev1[1].profile_id = 255
+    dev1[1].device_type = 255
+    dev1.model = "model1"
+    dev1.manufacturer = "manufacturer1"
+    dev1[1].add_input_cluster(3)
+    dev1[1].add_output_cluster(6)
+
+    dev2 = zigpy.device.Device(
+        app_mock, t.EUI64.convert("22:22:22:22:22:22:22:22"), 0x5678
+    )
+    dev2.add_endpoint(1)
+    dev2[1].profile_id = 255
+    dev2[1].device_type = 255
+    dev2.model = "model2"
+    dev2.manufacturer = "manufacturer2"
+    dev2[1].add_input_cluster(3)
+    dev2[1].add_output_cluster(6)
+
+    registry = zigpy.quirks.DEVICE_REGISTRY
+
+    assert not registry.registry_v1.get("manufacturer1", {}).get("model1", [])
+    assert not registry.registry_v2.get(("manufacturer2", "model2"), set())
+
+    load_quirks()
+
+    assert registry.registry_v1.get("manufacturer1", {}).get("model1", [])
+    assert registry.registry_v2.get(("manufacturer2", "model2"), set())
+
+    assert type(registry.get_device(dev1)).__name__ == "TestQuirk1"
+    assert registry.get_device(dev2).quirk_metadata.quirk_file.name == "quirk2.py"
+
+    # Only quirks from the passed directory are purged so this is a no-op
+    registry.purge_custom_quirks(tmp_path / "some_other_dir")
+    assert registry.registry_v1.get("manufacturer1", {}).get("model1", [])
+    assert registry.registry_v2.get(("manufacturer2", "model2"), set())
+
+    # Now we really remove them
+    registry.purge_custom_quirks(tmp_path)
+    assert not registry.registry_v1.get("manufacturer1", {}).get("model1", [])
+    assert not registry.registry_v2.get(("manufacturer2", "model2"), set())
+
+    assert registry.get_device(dev1) is dev1
+    assert registry.get_device(dev2) is dev2
