@@ -12,9 +12,7 @@ import zigpy.zdo.types as zdo_t
 
 @pytest.fixture
 def expose_global():
-    """
-    `typing.get_type_hints` does not work for types defined within functions
-    """
+    """`typing.get_type_hints` does not work for types defined within functions"""
 
     objects = []
 
@@ -394,12 +392,12 @@ def test_struct_subclass_extension(data):
 
     try:
         ts1, remaining1 = TestStructSubclass.deserialize(data)
-    except Exception as e:
+    except Exception as e:  # noqa: BLE001
         error1 = e
 
     try:
         ts2, remaining2 = TestCombinedStruct.deserialize(data)
-    except Exception as e:
+    except Exception as e:  # noqa: BLE001
         error2 = e
 
     assert (error1 and error2) or (not error1 and not error2)
@@ -624,7 +622,7 @@ def test_bitstruct_nesting(expose_global):
     assert InnerBitStruct.deserialize(inner.serialize() + b"asd") == (inner, b"asd")
 
     s = OuterStruct(foo=b"asd", bar=inner, asd=0xFF)
-    assert s.serialize() == b"\x03asd" + bytes([0b111_0_010_1]) + b"\xFF"
+    assert s.serialize() == b"\x03asd" + bytes([0b111_0_010_1]) + b"\xff"
 
     s2, remaining = OuterStruct.deserialize(s.serialize() + b"test")
     assert remaining == b"test"
@@ -644,7 +642,7 @@ def test_bitstruct_misaligned():
     assert s == s2
 
     with pytest.raises(ValueError):
-        TestStruct.deserialize(b"\xFF")
+        TestStruct.deserialize(b"\xff")
 
 
 def test_non_byte_sized_struct():
@@ -906,13 +904,55 @@ def test_int_comparison_non_int(expose_global):
     )
 
     with pytest.raises(TypeError):
-        fw_ver < 0
+        fw_ver < 0  # noqa: B015
 
     with pytest.raises(TypeError):
-        fw_ver <= 0
+        fw_ver <= 0  # noqa: B015
 
     with pytest.raises(TypeError):
-        fw_ver > 0
+        fw_ver > 0  # noqa: B015
 
     with pytest.raises(TypeError):
-        fw_ver >= 0
+        fw_ver >= 0  # noqa: B015
+
+
+def test_frozen_struct():
+    class OuterStruct(t.Struct):
+        class InnerStruct(t.Struct):
+            b: t.uint8_t
+            c: t.uint8_t
+
+        a: t.uint8_t
+        inner: None = t.StructField(type=InnerStruct)
+        d: t.uint8_t
+        e: t.uint16_t
+
+    struct = OuterStruct(a=1, inner=OuterStruct.InnerStruct(b=2, c=3), d=4)
+    frozen = struct.freeze()
+
+    assert "frozen" not in repr(struct)
+    assert "frozen" in repr(frozen)
+
+    with pytest.raises(TypeError, match="Unhashable type"):
+        hash(struct)
+
+    # Setting attributes has no effect
+    assert frozen.a == 1
+    assert frozen.inner.b == 2
+
+    with pytest.raises(AttributeError):
+        frozen.a = 2
+
+    with pytest.raises(AttributeError):
+        frozen.inner.b = 5
+
+    assert frozen.a == 1
+    assert frozen.inner.b == 2
+
+    assert {frozen: 2}[frozen] == 2
+    assert {frozen, frozen} == {frozen}
+    assert frozen == frozen.replace(a=1)
+    assert {frozen, frozen, frozen.replace(a=1), frozen.replace(a=2)} == {
+        frozen,
+        frozen.replace(a=2),
+    }
