@@ -1,17 +1,18 @@
 from __future__ import annotations
 
 import asyncio
+from typing import Any
 from unittest import mock
+from unittest.mock import AsyncMock, MagicMock, patch, sentinel
 
 import pytest
 
+from zigpy import zcl
 import zigpy.device
 import zigpy.endpoint
 import zigpy.types as t
-import zigpy.zcl as zcl
-import zigpy.zcl.foundation as foundation
-
-from .async_mock import AsyncMock, MagicMock, int_sentinel, patch, sentinel
+from zigpy.zcl import foundation
+from zigpy.zcl.clusters.general import Ota
 
 DEFAULT_TSN = 123
 
@@ -94,6 +95,8 @@ def cluster_by_id():
         epmock = MagicMock()
         epmock._device.get_sequence.return_value = DEFAULT_TSN
         epmock.device.get_sequence.return_value = DEFAULT_TSN
+        epmock.device.zdo.bind = AsyncMock()
+        epmock.device.zdo.unbind = AsyncMock()
         epmock.request = AsyncMock()
         epmock.reply = AsyncMock()
         return zcl.Cluster.from_id(epmock, cluster_id)
@@ -110,70 +113,164 @@ def cluster(cluster_by_id):
 def client_cluster():
     epmock = AsyncMock()
     epmock.device.get_sequence = MagicMock(return_value=DEFAULT_TSN)
-    return zcl.Cluster.from_id(epmock, 3)
+    return Ota(epmock)
 
 
 async def test_request_general(cluster):
-    await cluster.request(True, 0, [])
+    await cluster.request(
+        general=True,
+        command_id=foundation.GENERAL_COMMANDS[
+            foundation.GeneralCommand.Read_Attributes
+        ].id,
+        schema=foundation.GENERAL_COMMANDS[
+            foundation.GeneralCommand.Read_Attributes
+        ].schema,
+        attribute_ids=[],
+    )
     assert cluster._endpoint.request.call_count == 1
 
 
 async def test_request_manufacturer(cluster):
-    await cluster.request(True, 0, [t.uint8_t], 1)
+    command = foundation.ZCLCommandDef(
+        name="test_command", id=0x00, schema={"param1": t.uint8_t}
+    ).with_compiled_schema()
+
+    await cluster.request(
+        general=True,
+        command_id=command.id,
+        schema=command.schema,
+        param1=1,
+    )
     assert cluster._endpoint.request.call_count == 1
-    org_size = len(cluster._endpoint.request.call_args[0][2])
-    await cluster.request(True, 0, [t.uint8_t], 1, manufacturer=1)
+
+    org_size = len(cluster._endpoint.request.mock_calls[0].kwargs["data"])
+    await cluster.request(
+        general=True,
+        command_id=command.id,
+        schema=command.schema,
+        param1=1,
+        manufacturer=1,
+    )
     assert cluster._endpoint.request.call_count == 2
-    assert org_size + 2 == len(cluster._endpoint.request.call_args[0][2])
+    assert org_size + 2 == len(cluster._endpoint.request.mock_calls[1].kwargs["data"])
 
 
 async def test_request_optional(cluster):
-    schema = [t.uint8_t, t.uint16_t, t.Optional(t.uint16_t), t.Optional(t.uint8_t)]
+    command = foundation.ZCLCommandDef(
+        name="test_command",
+        id=0x00,
+        schema={
+            "param1": t.uint8_t,
+            "param2": t.uint16_t,
+            "param3?": t.uint16_t,
+            "param4?": t.uint8_t,
+        },
+    ).with_compiled_schema()
+
     cluster.endpoint.request = AsyncMock()
 
     with pytest.raises(ValueError):
-        await cluster.request(True, 0, schema)
+        await cluster.request(
+            general=True,
+            command_id=command.id,
+            schema=command.schema,
+        )
 
     assert cluster._endpoint.request.call_count == 0
     cluster._endpoint.request.reset_mock()
 
     with pytest.raises(ValueError):
-        await cluster.request(True, 0, schema, 1)
+        await cluster.request(
+            general=True,
+            command_id=command.id,
+            schema=command.schema,
+            param1=1,
+        )
 
     assert cluster._endpoint.request.call_count == 0
     cluster._endpoint.request.reset_mock()
 
-    await cluster.request(True, 0, schema, 1, 2)
+    await cluster.request(
+        general=True,
+        command_id=command.id,
+        schema=command.schema,
+        param1=1,
+        param2=2,
+    )
     assert cluster._endpoint.request.call_count == 1
     cluster._endpoint.request.reset_mock()
 
-    await cluster.request(True, 0, schema, 1, 2, 3)
+    await cluster.request(
+        general=True,
+        command_id=command.id,
+        schema=command.schema,
+        param1=1,
+        param2=2,
+        param3=3,
+    )
     assert cluster._endpoint.request.call_count == 1
     cluster._endpoint.request.reset_mock()
 
-    await cluster.request(True, 0, schema, 1, 2, 3, 4)
+    await cluster.request(
+        general=True,
+        command_id=command.id,
+        schema=command.schema,
+        param1=1,
+        param2=2,
+        param3=3,
+        param4=4,
+    )
     assert cluster._endpoint.request.call_count == 1
     cluster._endpoint.request.reset_mock()
 
     with pytest.raises(TypeError):
-        await cluster.request(True, 0, schema, 1, 2, 3, 4, 5)
+        await cluster.request(
+            general=True,
+            command_id=command.id,
+            schema=command.schema,
+            param1=1,
+            param2=2,
+            param3=3,
+            param4=4,
+            param5=5,
+        )
 
     assert cluster._endpoint.request.call_count == 0
     cluster._endpoint.request.reset_mock()
 
 
 async def test_reply_general(cluster):
-    await cluster.reply(False, 0, [])
+    command = foundation.ZCLCommandDef(
+        name="test_command", id=0x00, schema={}
+    ).with_compiled_schema()
+
+    await cluster.reply(general=False, command_id=command.id, schema=command.schema)
     assert cluster._endpoint.reply.call_count == 1
 
 
 async def test_reply_manufacturer(cluster):
-    await cluster.reply(False, 0, [t.uint8_t], 1)
+    command = foundation.ZCLCommandDef(
+        name="test_command",
+        id=0x00,
+        schema={
+            "param1": t.uint8_t,
+        },
+    ).with_compiled_schema()
+
+    await cluster.reply(
+        general=False, command_id=command.id, schema=command.schema, param1=1
+    )
     assert cluster._endpoint.reply.call_count == 1
-    org_size = len(cluster._endpoint.reply.call_args[0][2])
-    await cluster.reply(False, 0, [t.uint8_t], 1, manufacturer=1)
+    org_size = len(cluster._endpoint.reply.mock_calls[0].kwargs["data"])
+    await cluster.reply(
+        general=False,
+        command_id=command.id,
+        schema=command.schema,
+        param1=1,
+        manufacturer=1,
+    )
     assert cluster._endpoint.reply.call_count == 2
-    assert org_size + 2 == len(cluster._endpoint.reply.call_args[0][2])
+    assert org_size + 2 == len(cluster._endpoint.reply.mock_calls[1].kwargs["data"])
 
 
 def test_attribute_report(cluster):
@@ -197,21 +294,10 @@ def test_attribute_report(cluster):
     cluster.handle_message(hdr, cmd)
     assert cluster._attr_cache[attr.attrid] == "manufacturer"
 
-    def mock_type(*args, **kwargs):
-        raise ValueError
-
-    with patch.dict(
-        cluster.attributes,
-        {0xAAAA: foundation.ZCLAttributeDef(id=0xAAAA, name="Name", type=mock_type)},
-    ):
-        attr.attrid = 0xAAAA
-        cluster.handle_message(hdr, cmd)
-        assert cluster._attr_cache[attr.attrid] == "manufacturer"
-
 
 def test_handle_request_unknown(cluster):
     hdr = MagicMock(auto_spec=foundation.ZCLHeader)
-    hdr.command_id = int_sentinel.command_id
+    hdr.command_id = 0x42
     hdr.frame_control.is_general = True
     hdr.frame_control.is_cluster = False
     cluster.listener_event = MagicMock()
@@ -229,10 +315,10 @@ def test_handle_request_unknown(cluster):
 
 def test_handle_cluster_request(cluster):
     hdr = MagicMock(auto_spec=foundation.ZCLHeader)
-    hdr.command_id = int_sentinel.command_id
+    hdr.command_id = 0x42
     hdr.frame_control.is_general = False
     hdr.frame_control.is_cluster = True
-    hdr.command_id.is_general = False
+
     cluster.listener_event = MagicMock()
     cluster._update_attribute = MagicMock()
     cluster.handle_cluster_general_request = MagicMock()
@@ -324,28 +410,6 @@ async def test_read_attributes_default_response(cluster):
     assert failure == {0: 0xC1, 5: 0xC1, 23: 0xC1}
 
 
-async def test_read_attributes_value_normalization_error(cluster):
-    async def mockrequest(
-        foundation, command, schema, args, manufacturer=None, **kwargs
-    ):
-        assert foundation is True
-        assert command == 0
-        rar5 = _mk_rar(5, "Model")
-        return [[rar5]]
-
-    def mock_type(*args, **kwargs):
-        raise ValueError
-
-    cluster.request = mockrequest
-    with patch.dict(
-        cluster.attributes,
-        {5: foundation.ZCLAttributeDef(id=5, name="Name", type=mock_type)},
-    ):
-        success, failure = await cluster.read_attributes(["model"], allow_cache=True)
-    assert failure == {}
-    assert success["model"] == "Model"
-
-
 async def test_item_access_attributes(cluster):
     cluster._attr_cache[5] = sentinel.model
 
@@ -415,8 +479,10 @@ async def test_write_unknown_attribute(cluster):
 
 async def test_write_attributes_wrong_type(cluster):
     with patch.object(cluster, "_write_attributes", new=AsyncMock()):
-        await cluster.write_attributes({18: 0x2222})
-        assert cluster._write_attributes.call_count == 1
+        with pytest.raises(ValueError):
+            await cluster.write_attributes({18: 0x2222})
+
+        assert cluster._write_attributes.call_count == 0
 
 
 async def test_write_attributes_raw(cluster):
@@ -428,28 +494,28 @@ async def test_write_attributes_raw(cluster):
 
 
 @pytest.mark.parametrize(
-    "cluster_id, attr, value, serialized",
-    (
+    ("cluster_id", "attr", "value", "serialized"),
+    [
         (0, "zcl_version", 0xAA, b"\x00\x00\x20\xaa"),
         (0, "model", "model x", b"\x05\x00\x42\x07model x"),
         (0, "device_enabled", True, b"\x12\x00\x10\x01"),
         (0, "alarm_mask", 0x55, b"\x13\x00\x18\x55"),
         (0x0202, "fan_mode", 0xDE, b"\x00\x00\x30\xde"),
-    ),
+    ],
 )
 async def test_write_attribute_types(
-    cluster_id, attr, value, serialized, cluster_by_id
+    cluster_id: int, attr: str, value: Any, serialized: bytes, cluster_by_id
 ):
     cluster = cluster_by_id(cluster_id)
     with patch.object(cluster.endpoint, "request", new=AsyncMock()):
         await cluster.write_attributes({attr: value})
         assert cluster._endpoint.reply.call_count == 0
         assert cluster._endpoint.request.call_count == 1
-        assert cluster.endpoint.request.call_args[0][2][3:] == serialized
+        assert cluster.endpoint.request.mock_calls[0].kwargs["data"][3:] == serialized
 
 
 @pytest.mark.parametrize(
-    "status", (foundation.Status.SUCCESS, foundation.Status.UNSUPPORTED_ATTRIBUTE)
+    "status", [foundation.Status.SUCCESS, foundation.Status.UNSUPPORTED_ATTRIBUTE]
 )
 async def test_write_attributes_cache_default_response(cluster, status):
     write_mock = AsyncMock(
@@ -464,14 +530,14 @@ async def test_write_attributes_cache_default_response(cluster, status):
 
 
 @pytest.mark.parametrize(
-    "attributes, result",
-    (
+    ("attributes", "result"),
+    [
         ({4: "manufacturer"}, b"\x00"),
         ({4: "manufacturer", 5: "model"}, b"\x00"),
         ({4: "manufacturer", 5: "model", 3: 12}, b"\x00"),
         ({4: "manufacturer", 5: "model"}, b"\x00\x00"),
         ({4: "manufacturer", 5: "model", 3: 12}, b"\x00\x00\x00"),
-    ),
+    ],
 )
 async def test_write_attributes_cache_success(cluster, attributes, result):
     listener = MagicMock()
@@ -490,8 +556,8 @@ async def test_write_attributes_cache_success(cluster, attributes, result):
 
 
 @pytest.mark.parametrize(
-    "attributes, result, failed",
-    (
+    ("attributes", "result", "failed"),
+    [
         ({4: "manufacturer"}, b"\x86\x04\x00", [4]),
         ({4: "manufacturer", 5: "model"}, b"\x86\x05\x00", [5]),
         ({4: "manufacturer", 5: "model"}, b"\x86\x04\x00\x86\x05\x00", [4, 5]),
@@ -510,7 +576,7 @@ async def test_write_attributes_cache_success(cluster, attributes, result):
             b"\x02\x04\x00\x86\x05\x00\x01\x03\x00",
             [4, 5, 3],
         ),
-    ),
+    ],
 )
 async def test_write_attributes_cache_failure(cluster, attributes, result, failed):
     listener = MagicMock()
@@ -538,63 +604,20 @@ async def test_write_attributes_cache_failure(cluster, attributes, result, faile
                 )
 
 
-async def test_read_attributes_response(cluster):
-    await cluster.read_attributes_rsp({0: 5})
-    assert cluster._endpoint.reply.call_count == 1
-    assert cluster._endpoint.request.call_count == 0
+async def test_bind(cluster):
+    result = await cluster.bind()
+
+    cluster._endpoint.device.zdo.bind.assert_called_with(cluster=cluster)
+    assert cluster._endpoint.device.zdo.bind.call_count == 1
+    assert result is cluster._endpoint.device.zdo.bind.return_value
 
 
-async def test_read_attributes_resp_unsupported(cluster):
-    await cluster.read_attributes_rsp({0: 5})
-    assert cluster._endpoint.reply.call_count == 1
-    assert cluster._endpoint.request.call_count == 0
-    orig_len = len(cluster._endpoint.reply.call_args[0][2])
+async def test_unbind(cluster):
+    result = await cluster.unbind()
 
-    await cluster.read_attributes_rsp({0: 5, 2: None})
-    assert cluster._endpoint.reply.call_count == 2
-    assert cluster._endpoint.request.call_count == 0
-    assert len(cluster._endpoint.reply.call_args[0][2]) == orig_len + 3
-
-
-async def test_read_attributes_resp_str(cluster):
-    await cluster.read_attributes_rsp({"hw_version": 32})
-    assert cluster._endpoint.reply.call_count == 1
-    assert cluster._endpoint.request.call_count == 0
-
-
-async def test_read_attributes_resp_exc(cluster):
-    with patch.object(foundation.DATA_TYPES, "pytype_to_datatype_id") as mck:
-        mck.side_effect = ValueError
-        await cluster.read_attributes_rsp({"hw_version": 32})
-    assert cluster._endpoint.reply.call_count == 1
-    assert cluster._endpoint.request.call_count == 0
-    assert cluster.endpoint.reply.call_args[0][2][-3:] == b"\x03\x00\x86"
-
-
-@pytest.mark.parametrize(
-    "cluster_id, attr, value, serialized",
-    (
-        (0, "zcl_version", 0xAA, b"\x00\x00\x00\x20\xaa"),
-        (0, "model", "model x", b"\x05\x00\x00\x42\x07model x"),
-        (0, "device_enabled", True, b"\x12\x00\x00\x10\x01"),
-        (0, "alarm_mask", 0x55, b"\x13\x00\x00\x18\x55"),
-        (0x0202, "fan_mode", 0xDE, b"\x00\x00\x00\x30\xde"),
-    ),
-)
-async def test_read_attribute_resp(cluster_id, attr, value, serialized, cluster_by_id):
-    cluster = cluster_by_id(cluster_id)
-    await cluster.read_attributes_rsp({attr: value})
-    assert cluster._endpoint.reply.call_count == 1
-    assert cluster._endpoint.request.call_count == 0
-    assert cluster.endpoint.reply.call_args[0][2][3:] == serialized
-
-
-def test_bind(cluster):
-    cluster.bind()
-
-
-def test_unbind(cluster):
-    cluster.unbind()
+    cluster._endpoint.device.zdo.unbind.assert_called_with(cluster=cluster)
+    assert cluster._endpoint.device.zdo.unbind.call_count == 1
+    assert result is cluster._endpoint.device.zdo.unbind.return_value
 
 
 async def test_configure_reporting(cluster):
@@ -609,13 +632,15 @@ async def test_configure_reporting_named(cluster):
 async def test_configure_reporting_wrong_named(cluster):
     with pytest.raises(ValueError):
         await cluster.configure_reporting("wrong_attr_name", 10, 20, 1)
-        assert cluster._endpoint.request.call_count == 0
+
+    assert cluster._endpoint.request.call_count == 0
 
 
 async def test_configure_reporting_wrong_attrid(cluster):
     with pytest.raises(ValueError):
         await cluster.configure_reporting(0xABCD, 10, 20, 1)
-        assert cluster._endpoint.request.call_count == 0
+
+    assert cluster._endpoint.request.call_count == 0
 
 
 async def test_configure_reporting_manuf():
@@ -649,33 +674,33 @@ async def test_configure_reporting_manuf():
 
 
 @pytest.mark.parametrize(
-    "cluster_id, attr, data_type",
-    (
+    ("cluster_id", "attr", "data_type"),
+    [
         (0, "zcl_version", 0x20),
         (0, "model", 0x42),
         (0, "device_enabled", 0x10),
         (0, "alarm_mask", 0x18),
         (0x0202, "fan_mode", 0x30),
-    ),
+    ],
 )
 async def test_configure_reporting_types(cluster_id, attr, data_type, cluster_by_id):
     cluster = cluster_by_id(cluster_id)
     await cluster.configure_reporting(attr, 0x1234, 0x2345, 0xAA)
     assert cluster._endpoint.reply.call_count == 0
     assert cluster._endpoint.request.call_count == 1
-    assert cluster.endpoint.request.call_args[0][2][6] == data_type
+    assert cluster.endpoint.request.mock_calls[0].kwargs["data"][6] == data_type
 
 
 async def test_command(cluster):
     await cluster.command(0x00)
     assert cluster._endpoint.request.call_count == 1
-    assert cluster._endpoint.request.call_args[0][1] == DEFAULT_TSN
+    assert cluster._endpoint.request.mock_calls[0].kwargs["sequence"] == DEFAULT_TSN
 
 
 async def test_command_override_tsn(cluster):
     await cluster.command(0x00, tsn=22)
     assert cluster._endpoint.request.call_count == 1
-    assert cluster._endpoint.request.call_args[0][1] == 22
+    assert cluster._endpoint.request.mock_calls[0].kwargs["sequence"] == 22
 
 
 async def test_command_attr(cluster):
@@ -684,7 +709,7 @@ async def test_command_attr(cluster):
 
 
 async def test_client_command_attr(client_cluster):
-    await client_cluster.identify_query_response(timeout=0)
+    await client_cluster.query_specific_file_response(status=foundation.Status.SUCCESS)
     assert client_cluster._endpoint.reply.call_count == 1
 
 
@@ -699,8 +724,13 @@ async def test_invalid_arguments_cluster_command(cluster):
 
 
 async def test_invalid_arguments_cluster_client_command(client_cluster):
-    with pytest.raises(TypeError):
-        await client_cluster.client_command(0, 0, 0)
+    with pytest.raises(ValueError):
+        await client_cluster.client_command(
+            command_id=Ota.ClientCommandDefs.upgrade_end_response.id,
+            manufacturer_code=0,
+            image_type=0,
+            # Missing: file_version, current_time, upgrade_time
+        )
 
 
 def test_name(cluster):
@@ -762,9 +792,9 @@ def test_handle_cluster_request_handler(cluster):
 async def test_handle_cluster_general_request_disable_default_rsp(endpoint):
     hdr, values = endpoint.deserialize(
         0,
-        b"\x18\xCD\x0A\x01\xFF\x42\x25\x01\x21\x95\x0B\x04\x21\xA8\x43\x05\x21\x36\x00"
-        b"\x06\x24\x02\x00\x05\x00\x00\x64\x29\xF8\x07\x65\x21\xD9\x0E\x66\x2B\x84\x87"
-        b"\x01\x00\x0A\x21\x00\x00",
+        b"\x18\xcd\x0a\x01\xff\x42\x25\x01\x21\x95\x0b\x04\x21\xa8\x43\x05\x21\x36\x00"
+        b"\x06\x24\x02\x00\x05\x00\x00\x64\x29\xf8\x07\x65\x21\xd9\x0e\x66\x2b\x84\x87"
+        b"\x01\x00\x0a\x21\x00\x00",
     )
     cluster = endpoint.in_clusters[0]
     p1 = patch.object(cluster, "_update_attribute")
@@ -803,12 +833,20 @@ async def test_write_attributes_undivided(cluster):
 
 
 async def test_configure_reporting_multiple(cluster):
-    await cluster.configure_reporting(3, 5, 15, 20, manufacturer=0x2345)
-    await cluster.configure_reporting_multiple({3: (5, 15, 20)}, manufacturer=0x2345)
+    await cluster.configure_reporting(
+        attribute=3,
+        min_interval=5,
+        max_interval=15,
+        reportable_change=20,
+        manufacturer=0x2345,
+    )
+    await cluster.configure_reporting_multiple(
+        attributes={3: (5, 15, 20)}, manufacturer=0x2345
+    )
     assert cluster.endpoint.request.call_count == 2
     assert (
-        cluster.endpoint.request.call_args_list[0][0][2]
-        == cluster.endpoint.request.call_args_list[1][0][2]
+        cluster.endpoint.request.mock_calls[0].kwargs["data"]
+        == cluster.endpoint.request.mock_calls[2].kwargs["data"]
     )
 
 
@@ -1009,7 +1047,7 @@ def test_zcl_command_duplicate_name_prevention():
 
 def test_zcl_attridx_deprecation(cluster):
     with pytest.deprecated_call():
-        cluster.attridx
+        cluster.attridx  # noqa: B018
 
     with pytest.deprecated_call():
         assert cluster.attridx is cluster.attributes_by_name
@@ -1032,7 +1070,7 @@ def test_zcl_response_type_tuple_like():
     assert req.off_wait_time == off_wait_time == req[2] == 2
 
     assert req == (0, 1, 2)
-    assert req == req
+    assert req == req  # noqa: PLR0124
     assert req == req.replace()
 
 
@@ -1051,14 +1089,14 @@ async def test_zcl_request_direction():
 
     # Input cluster
     await ep.in_clusters[zcl.clusters.general.OnOff.cluster_id].on()
-    hdr1, _ = foundation.ZCLHeader.deserialize(ep.request.mock_calls[0].args[2])
+    hdr1, _ = foundation.ZCLHeader.deserialize(ep.request.mock_calls[0].kwargs["data"])
     assert hdr1.direction == foundation.Direction.Client_to_Server
 
     ep.request.reset_mock()
 
     # Output cluster
     await ep.out_clusters[zcl.clusters.general.OnOff.cluster_id].on()
-    hdr2, _ = foundation.ZCLHeader.deserialize(ep.request.mock_calls[0].args[2])
+    hdr2, _ = foundation.ZCLHeader.deserialize(ep.request.mock_calls[0].kwargs["data"])
     assert hdr2.direction == foundation.Direction.Server_to_Client
 
     # Color cluster that also uses `direction` as a kwarg
@@ -1118,3 +1156,115 @@ async def test_zcl_reply_direction(app_mock):
     # The direction is correct
     packet_hdr, _ = foundation.ZCLHeader.deserialize(packet.data.serialize())
     assert packet_hdr.direction == foundation.Direction.Client_to_Server
+
+
+async def test_zcl_cluster_definition_backwards_compatibility():
+    class TestCluster(zcl.Cluster):
+        cluster_id = 0xABCD
+        ep_attribute = "test_cluster"
+
+        attributes = {
+            0x1234: ("attribute", t.uint8_t),
+            0x1235: ("attribute2", t.uint32_t, True),
+        }
+
+        server_commands = {
+            0x00: ("server_command", (t.uint8_t,), True),
+        }
+
+        client_commands = {
+            0x01: ("client_command", (t.uint8_t, t.uint16_t), False),
+        }
+
+    assert TestCluster.cluster_id == 0xABCD
+
+    assert TestCluster.AttributeDefs.attribute.id == 0x1234
+    assert TestCluster.AttributeDefs.attribute.type == t.uint8_t
+    assert TestCluster.AttributeDefs.attribute.is_manufacturer_specific is False
+
+    assert TestCluster.AttributeDefs.attribute2.id == 0x1235
+    assert TestCluster.AttributeDefs.attribute2.type == t.uint32_t
+    assert TestCluster.AttributeDefs.attribute2.is_manufacturer_specific is True
+
+    assert TestCluster.ServerCommandDefs.server_command.id == 0x00
+    assert len(TestCluster.ServerCommandDefs.server_command.schema.fields) == 1
+    assert (
+        TestCluster.ServerCommandDefs.server_command.schema.fields.param1.type
+        == t.uint8_t
+    )
+
+    assert TestCluster.ClientCommandDefs.client_command.id == 0x01
+    assert len(TestCluster.ClientCommandDefs.client_command.schema.fields) == 2
+    assert (
+        TestCluster.ClientCommandDefs.client_command.schema.fields.param1.type
+        == t.uint8_t
+    )
+    assert (
+        TestCluster.ClientCommandDefs.client_command.schema.fields.param2.type
+        == t.uint16_t
+    )
+
+
+async def test_zcl_cluster_definition_invalid_name():
+    # This is fine
+    class TestCluster(zcl.Cluster):
+        cluster_id = 0xABCD
+        ep_attribute = "test_cluster"
+
+        class AttributeDefs(zcl.BaseAttributeDefs):
+            upgrade_server_id = foundation.ZCLAttributeDef(
+                name="upgrade_server_id",
+                id=0x0000,
+                type=t.EUI64,
+                access="r",
+                mandatory=True,
+            )
+
+        class ServerCommandDefs(zcl.BaseCommandDefs):
+            upgrade_end = foundation.ZCLCommandDef(
+                name="upgrade_end",
+                id=0x06,
+                schema={
+                    "status": foundation.Status,
+                    "manufacturer_code": t.uint16_t,
+                    "image_type": t.uint16_t,
+                    "file_version": t.uint32_t,
+                },
+                direction=foundation.Direction.Client_to_Server,
+            )
+
+    # This is not
+    with pytest.raises(TypeError):
+
+        class TestCluster(zcl.Cluster):
+            cluster_id = 0xABCD
+            ep_attribute = "test_cluster"
+
+            class AttributeDefs(zcl.BaseAttributeDefs):
+                upgrade_server_id = foundation.ZCLAttributeDef(
+                    name="some_other_name",
+                    id=0x0000,
+                    type=t.EUI64,
+                    access="r",
+                    mandatory=True,
+                )
+
+    # Nor is this
+    with pytest.raises(TypeError):
+
+        class TestCluster(zcl.Cluster):
+            cluster_id = 0xABCD
+            ep_attribute = "test_cluster"
+
+            class ServerCommandDefs(zcl.BaseCommandDefs):
+                upgrade_end = foundation.ZCLCommandDef(
+                    name="some_other_name",
+                    id=0x06,
+                    schema={
+                        "status": foundation.Status,
+                        "manufacturer_code": t.uint16_t,
+                        "image_type": t.uint16_t,
+                        "file_version": t.uint32_t,
+                    },
+                    direction=foundation.Direction.Client_to_Server,
+                )
