@@ -1,16 +1,22 @@
+from __future__ import annotations
+
 import asyncio
+from datetime import datetime, timezone
 import re
+from typing import Any
+from zoneinfo import ZoneInfo
 
 import pytest
 
-from zigpy import device
+from zigpy import device, types, zcl
 import zigpy.endpoint
-import zigpy.types as types
-import zigpy.zcl as zcl
+from zigpy.ota import OtaImagesResult
+from zigpy.zcl import foundation
+from zigpy.zcl.clusters.general import Basic, Ota, Time
 import zigpy.zcl.clusters.security as sec
 from zigpy.zdo import types as zdo_t
 
-from .async_mock import AsyncMock, MagicMock, patch, sentinel
+from .async_mock import AsyncMock, MagicMock, call, patch, sentinel
 
 IMAGE_SIZE = 0x2345
 IMAGE_OFFSET = 0x2000
@@ -24,7 +30,7 @@ def test_registry():
 
 
 def test_attributes():
-    for _cluster_id, cluster in zcl.Cluster._registry.items():
+    for cluster in zcl.Cluster._registry.values():
         for attrid, attr in cluster.attributes.items():
             assert 0 <= attrid <= 0xFFFF
             assert isinstance(attr, zcl.foundation.ZCLAttributeDef)
@@ -36,7 +42,7 @@ def test_attributes():
 
 
 def _test_commands(cmdattr):
-    for _cluster_id, cluster in zcl.Cluster._registry.items():
+    for cluster in zcl.Cluster._registry.values():
         for cmdid, cmdspec in getattr(cluster, cmdattr).items():
             assert 0 <= cmdid <= 0xFF
 
@@ -59,7 +65,7 @@ def test_client_commands():
 
 def test_ep_attributes():
     seen = set()
-    for _cluster_id, cluster in zcl.Cluster._registry.items():
+    for cluster in zcl.Cluster._registry.values():
         assert isinstance(cluster.ep_attribute, str)
         assert re.match(r"^[a-z_][a-z0-9_]*$", cluster.ep_attribute)
         assert cluster.ep_attribute not in seen
@@ -69,58 +75,180 @@ def test_ep_attributes():
         assert not hasattr(ep, cluster.ep_attribute)
 
 
+async def read_attributes(cluster, attribute_ids: list[int]) -> dict[int, Any]:
+    schema = foundation.GENERAL_COMMANDS[
+        foundation.GeneralCommand.Read_Attributes
+    ].schema
+    hdr, _ = cluster._create_request(
+        general=True,
+        command_id=foundation.GeneralCommand.Read_Attributes,
+        schema=schema,
+        disable_default_response=False,
+        direction=foundation.Direction.Client_to_Server,
+        args=(),
+        kwargs={"attribute_ids": attribute_ids},
+    )
+
+    command = schema(attribute_ids=attribute_ids)
+
+    with patch.object(cluster, "reply") as reply_mock:
+        cluster.handle_message(hdr, command)
+        call = reply_mock.mock_calls[0]
+
+    return call.args[2](call.args[3])
+
+
+async def test_basic_cluster():
+    ep = MagicMock()
+    ep.reply = AsyncMock()
+
+    cluster = Basic(ep)
+
+    rsp = await read_attributes(
+        cluster,
+        [
+            Basic.AttributeDefs.zcl_version.id,
+            Basic.AttributeDefs.power_source.id,
+            Basic.AttributeDefs.serial_number.id,
+        ],
+    )
+
+    assert rsp.status_records[0] == foundation.ReadAttributeRecord(
+        attrid=Basic.AttributeDefs.zcl_version.id,
+        status=foundation.Status.SUCCESS,
+        value=foundation.TypeValue(
+            type=foundation.DataTypeId.uint8,
+            value=8,
+        ),
+    )
+
+    assert rsp.status_records[1] == foundation.ReadAttributeRecord(
+        attrid=Basic.AttributeDefs.power_source.id,
+        status=foundation.Status.SUCCESS,
+        value=foundation.TypeValue(
+            type=foundation.DataTypeId.enum8,
+            value=Basic.PowerSource.DC_Source,
+        ),
+    )
+
+    assert rsp.status_records[2] == foundation.ReadAttributeRecord(
+        attrid=Basic.AttributeDefs.serial_number.id,
+        status=foundation.Status.UNSUPPORTED_ATTRIBUTE,
+    )
+
+
 async def test_time_cluster():
     ep = MagicMock()
     ep.reply = AsyncMock()
-    t = zcl.Cluster._registry[0x000A](ep)
 
-    hdr_general = zcl.foundation.ZCLHeader.general
-    tsn = 123
+    cluster = Time(ep)
 
-    t.handle_message(hdr_general(tsn, 1), [[0]])
-    await asyncio.sleep(0.01)
-    assert ep.reply.call_count == 0
+    Read_Attributes_rsp = foundation.GENERAL_COMMANDS[
+        foundation.GeneralCommand.Read_Attributes_rsp
+    ].schema
 
-    t.handle_message(hdr_general(tsn, 0), [[0]])
-    await asyncio.sleep(0.01)
-    assert ep.reply.call_count == 1
-    assert ep.reply.call_args[0][2][3] == 0
+    # Datetime objects need to be subclassed to be patched so we may as well implement
+    # the patches directly
 
-    t.handle_message(hdr_general(tsn, 0), [[1]])
-    await asyncio.sleep(0.01)
-    assert ep.reply.call_count == 2
-    assert ep.reply.call_args[0][2][3] == 1
+    class PatchedDatetime(datetime):
+        _fake_now = datetime(
+            2000, 1, 2, 0, 0, 0, tzinfo=ZoneInfo("America/Los_Angeles")
+        )
 
-    t.handle_message(hdr_general(tsn, 0), [[2]])
-    await asyncio.sleep(0.01)
-    assert ep.reply.call_count == 3
-    assert ep.reply.call_args[0][2][3] == 2
+        def astimezone(self):
+            return self.replace(tzinfo=self._fake_now.tzinfo)
 
-    t.handle_message(hdr_general(tsn, 0), [[0, 1, 2]])
-    await asyncio.sleep(0.01)
-    assert ep.reply.call_count == 4
-    assert ep.reply.call_args[0][2][3] == 0
+        @classmethod
+        def now(cls, tzinfo=None):
+            if tzinfo is None:
+                return cls(
+                    cls._fake_now.year,
+                    cls._fake_now.month,
+                    cls._fake_now.day,
+                    cls._fake_now.hour,
+                    cls._fake_now.minute,
+                    cls._fake_now.second,
+                )
+            else:
+                assert tzinfo is timezone.utc
+                return (
+                    cls(
+                        cls._fake_now.year,
+                        cls._fake_now.month,
+                        cls._fake_now.day,
+                        cls._fake_now.hour,
+                        cls._fake_now.minute,
+                        cls._fake_now.second,
+                        tzinfo=tzinfo,
+                    )
+                    - cls._fake_now.utcoffset()
+                )
 
-    t.handle_message(hdr_general(tsn, 0), [[7]])
-    await asyncio.sleep(0.01)
-    assert ep.reply.call_count == 5
-    assert ep.reply.call_args[0][2][3] == 7
+    with patch("zigpy.zcl.clusters.general.datetime", PatchedDatetime):
+        # Supported attributes
+        rsp1 = await read_attributes(
+            cluster,
+            [
+                Time.AttributeDefs.time.id,
+                Time.AttributeDefs.time_status.id,
+                Time.AttributeDefs.time_zone.id,
+                Time.AttributeDefs.local_time.id,
+            ],
+        )
 
+    assert rsp1.status_records[0] == foundation.ReadAttributeRecord(
+        attrid=Time.AttributeDefs.time.id,
+        status=foundation.Status.SUCCESS,
+        value=foundation.TypeValue(
+            type=foundation.DataTypeId.UTC,
+            # One day from the epoch, plus time zone offset
+            value=24 * 60 * 60 + 8 * 60 * 60,
+        ),
+    )
 
-async def test_time_cluster_unsupported():
-    ep = MagicMock()
-    ep.reply = AsyncMock()
-    t = zcl.Cluster._registry[0x000A](ep)
+    assert rsp1.status_records[1] == foundation.ReadAttributeRecord(
+        attrid=Time.AttributeDefs.time_status.id,
+        status=foundation.Status.SUCCESS,
+        value=foundation.TypeValue(
+            type=foundation.DataTypeId.map8,
+            value=(
+                Time.TimeStatus.Master
+                | Time.TimeStatus.Synchronized
+                | Time.TimeStatus.Master_for_Zone_and_DST
+            ),
+        ),
+    )
 
-    hdr_general = zcl.foundation.ZCLHeader.general
-    tsn = 123
+    assert rsp1.status_records[2] == foundation.ReadAttributeRecord(
+        attrid=Time.AttributeDefs.time_zone.id,
+        status=foundation.Status.SUCCESS,
+        value=foundation.TypeValue(
+            type=foundation.DataTypeId.int32,
+            # Time zone offset
+            value=-(8 * 60 * 60),
+        ),
+    )
 
-    t.handle_cluster_general_request(hdr_general(tsn, 0), [[199, 128]])
+    assert rsp1.status_records[3] == foundation.ReadAttributeRecord(
+        attrid=Time.AttributeDefs.local_time.id,
+        status=foundation.Status.SUCCESS,
+        value=foundation.TypeValue(
+            type=foundation.DataTypeId.uint32,
+            # One day from the epoch, as on the clock
+            value=24 * 60 * 60,
+        ),
+    )
 
-    await asyncio.sleep(0.01)
-
-    assert ep.reply.call_count == 1
-    assert ep.reply.call_args[0][2][-6:] == b"\xc7\x00\x86\x80\x00\x86"
+    # Unsupported
+    rsp2 = await read_attributes(cluster, [0xABCD])
+    assert rsp2 == Read_Attributes_rsp(
+        status_records=[
+            foundation.ReadAttributeRecord(
+                attrid=0xABCD,
+                status=foundation.Status.UNSUPPORTED_ATTRIBUTE,
+            )
+        ]
+    )
 
 
 @pytest.fixture
@@ -139,127 +267,109 @@ def dev(monkeypatch, app_mock):
 @pytest.fixture
 def ota_cluster(dev):
     ep = dev.add_endpoint(1)
-    # ep.device.application.ota = MagicMock(spec_set=ota.OTA)
 
     cluster = zcl.Cluster._registry[0x0019](ep)
 
-    with patch.object(cluster, "reply", AsyncMock()), patch.object(
-        cluster, "request", AsyncMock()
+    with (
+        patch.object(cluster, "reply", AsyncMock()),
+        patch.object(cluster, "request", AsyncMock()),
     ):
         yield cluster
 
 
-async def test_ota_handle_cluster_req(ota_cluster):
-    ota_cluster._handle_cluster_request = AsyncMock()
-
-    hdr = zigpy.zcl.foundation.ZCLHeader.cluster(123, 0x00)
-    ota_cluster.handle_cluster_request(hdr, sentinel.args)
-    assert ota_cluster._handle_cluster_request.call_count == 1
-
-
 async def test_ota_handle_cluster_req_wrapper(ota_cluster, caplog):
     ota_cluster._handle_query_next_image = AsyncMock()
-    ota_cluster._handle_image_block = AsyncMock()
-    ota_cluster._handle_upgrade_end = AsyncMock()
 
     hdr = zigpy.zcl.foundation.ZCLHeader.cluster(123, 0x01)
-    await ota_cluster._handle_cluster_request(hdr, [sentinel.args])
+    ota_cluster.handle_cluster_request(hdr, [sentinel.args])
     assert ota_cluster._handle_query_next_image.call_count == 1
-    assert ota_cluster._handle_query_next_image.call_args[0][0] == sentinel.args
-    assert ota_cluster._handle_image_block.call_count == 0
-    assert ota_cluster._handle_upgrade_end.call_count == 0
+    assert ota_cluster._handle_query_next_image.mock_calls[0].args == (
+        hdr,
+        [sentinel.args],
+    )
     ota_cluster._handle_query_next_image.reset_mock()
-    ota_cluster._handle_image_block.reset_mock()
-    ota_cluster._handle_upgrade_end.reset_mock()
 
     # This command isn't currently handled
     hdr.command_id = 0x08
-    await ota_cluster._handle_cluster_request(hdr, [sentinel.just_args])
+    ota_cluster.handle_cluster_request(hdr, [sentinel.just_args])
     assert ota_cluster._handle_query_next_image.call_count == 0
-    assert ota_cluster._handle_image_block.call_count == 0
-    assert ota_cluster._handle_upgrade_end.call_count == 0
-
-    # This command doesn't exist
-    hdr.command_id = 0x28
-    await ota_cluster._handle_cluster_request(hdr, [sentinel.just_args])
-    assert ota_cluster._handle_query_next_image.call_count == 0
-    assert ota_cluster._handle_image_block.call_count == 0
-    assert ota_cluster._handle_upgrade_end.call_count == 0
-    assert "Unknown OTA command id" in caplog.text
 
 
-def _ota_next_image(cluster, has_image=True, upgradeable=False):
-    async def get_ota_mock(*args):
-        if upgradeable:
-            img = MagicMock()
-            img.should_update = MagicMock(return_value=True)
-            img.key.manufacturer_id = sentinel.manufacturer_id
-            img.key.image_type = sentinel.image_type
-            img.version = sentinel.image_version
-            img.header.image_size = sentinel.image_size
-        elif has_image:
-            img = MagicMock()
-            img.should_update.return_value = False
-        else:
-            img = None
-        return img
+async def test_ota_handle_query_next_image(ota_cluster):
+    dev = ota_cluster.endpoint.device
 
-    cluster.endpoint.device.application.ota.get_ota_image = MagicMock(
-        side_effect=get_ota_mock
-    )
-    return cluster._handle_query_next_image(
-        sentinel.field_ctrl,
-        sentinel.manufacturer_id,
-        sentinel.image_type,
-        sentinel.current_file_version,
-        sentinel.hw_version,
-        tsn=0x21,
-    )
-
-
-async def test_ota_handle_query_next_image_no_img(ota_cluster):
     ota_cluster.query_next_image_response = AsyncMock()
-    ota_cluster.endpoint.device.ota_in_progress = False
+    dev.ota_in_progress = False
 
-    await _ota_next_image(ota_cluster, has_image=False, upgradeable=False)
-    assert ota_cluster.query_next_image_response.call_count == 1
-    assert (
-        ota_cluster.query_next_image_response.call_args[0][0]
-        == zcl.foundation.Status.NO_IMAGE_AVAILABLE
+    listener = MagicMock()
+    dev.add_listener(listener)
+
+    # TODO: get rid of `sentinel` and mock the actual command
+    hdr = zigpy.zcl.foundation.ZCLHeader.cluster(
+        tsn=0x12, command_id=Ota.ServerCommandDefs.query_next_image.id
     )
-    assert len(ota_cluster.query_next_image_response.call_args[0]) == 1
+    cmd = MagicMock()
 
-
-async def test_ota_handle_query_next_image_not_upgradeable(ota_cluster):
-    ota_cluster.query_next_image_response = AsyncMock()
-    ota_cluster.endpoint.device.ota_in_progress = False
-
-    await _ota_next_image(ota_cluster, has_image=True, upgradeable=False)
-    assert ota_cluster.query_next_image_response.call_count == 1
-    assert (
-        ota_cluster.query_next_image_response.call_args[0][0]
-        == zcl.foundation.Status.NO_IMAGE_AVAILABLE
+    # No image is available
+    dev.application.ota.get_ota_images = AsyncMock(
+        return_value=OtaImagesResult(upgrades=(), downgrades=())
     )
-    assert len(ota_cluster.query_next_image_response.call_args[0]) == 1
+    ota_cluster.handle_cluster_request(hdr, cmd)
+    await asyncio.sleep(0)
 
+    assert ota_cluster.query_next_image_response.mock_calls == [
+        call(zcl.foundation.Status.NO_IMAGE_AVAILABLE, tsn=hdr.tsn)
+    ]
+    assert listener.device_ota_image_query_result.mock_calls == [
+        call(OtaImagesResult(upgrades=(), downgrades=()), cmd)
+    ]
 
-async def test_ota_handle_query_next_image_upgradeable(ota_cluster):
-    ota_cluster.query_next_image_response = AsyncMock()
-    ota_cluster.endpoint.device.ota_in_progress = False
+    ota_cluster.query_next_image_response.reset_mock()
+    listener.device_ota_image_query_result.reset_mock()
 
-    class Listener:
-        device_ota_update_available = MagicMock()
-
-    listener = Listener()
-    ota_cluster.endpoint.device.add_listener(listener)
-    await _ota_next_image(ota_cluster, has_image=True, upgradeable=True)
-    assert ota_cluster.query_next_image_response.call_count == 1
-    assert (
-        ota_cluster.query_next_image_response.call_args[0][0]
-        == zcl.foundation.Status.NO_IMAGE_AVAILABLE
+    # Now one is available
+    img = MagicMock()
+    dev.application.ota.get_ota_images = AsyncMock(
+        return_value=OtaImagesResult(upgrades=(img,), downgrades=())
     )
-    assert len(ota_cluster.query_next_image_response.call_args[0]) == 1
-    assert listener.device_ota_update_available.call_count == 1
+    ota_cluster.handle_cluster_request(hdr, cmd)
+    await asyncio.sleep(0)
+
+    assert ota_cluster.query_next_image_response.mock_calls == [
+        call(zcl.foundation.Status.NO_IMAGE_AVAILABLE, tsn=hdr.tsn)
+    ]
+    assert listener.device_ota_image_query_result.mock_calls == [
+        call(OtaImagesResult(upgrades=(img,), downgrades=()), cmd)
+    ]
+
+
+async def test_ota_handle_image_block_req(ota_cluster):
+    dev = ota_cluster.endpoint.device
+
+    ota_cluster.image_block_response = AsyncMock()
+    dev.ota_in_progress = False
+
+    hdr = zigpy.zcl.foundation.ZCLHeader.cluster(
+        tsn=0x12, command_id=Ota.ServerCommandDefs.image_block.id
+    )
+    cmd = MagicMock()
+
+    # Stop the upgrade, none is in progress
+    ota_cluster.handle_cluster_request(hdr, cmd)
+    await asyncio.sleep(0)
+
+    assert ota_cluster.image_block_response.mock_calls == [
+        call(zcl.foundation.Status.ABORT, tsn=hdr.tsn)
+    ]
+
+    ota_cluster.image_block_response.reset_mock()
+
+    # If we flip the progress flag, send nothing
+    dev.ota_in_progress = True
+    ota_cluster.handle_cluster_request(hdr, cmd)
+    await asyncio.sleep(0)
+
+    assert ota_cluster.image_block_response.mock_calls == []
 
 
 def test_ias_zone_type():
@@ -302,8 +412,8 @@ def test_basic_cluster_power_source():
 
 
 @pytest.mark.parametrize(
-    "raw, mode, name",
-    (
+    ("raw", "mode", "name"),
+    [
         (0x00, 0, "Stop"),
         (0x01, 0, "Stop"),
         (0x02, 0, "Stop"),
@@ -312,7 +422,7 @@ def test_basic_cluster_power_source():
         (0x31, 3, "Emergency"),
         (0x32, 3, "Emergency"),
         (0x33, 3, "Emergency"),
-    ),
+    ],
 )
 def test_security_iaswd_warning_mode(raw, mode, name):
     """Test warning command class of IasWD cluster."""
@@ -395,8 +505,8 @@ def test_security_iaswd_warning_siren():
 
 
 @pytest.mark.parametrize(
-    "raw, mode, name",
-    (
+    ("raw", "mode", "name"),
+    [
         (0x00, 0, "Armed"),
         (0x01, 0, "Armed"),
         (0x02, 0, "Armed"),
@@ -405,7 +515,7 @@ def test_security_iaswd_warning_siren():
         (0x11, 1, "Disarmed"),
         (0x12, 1, "Disarmed"),
         (0x13, 1, "Disarmed"),
-    ),
+    ],
 )
 def test_security_iaswd_squawk_mode(raw, mode, name):
     """Test squawk command class of IasWD cluster."""
@@ -540,3 +650,20 @@ def test_ota_image_block_field_control():
     )
 
     assert response.request_node_addr is None
+
+
+def test_general_analog_in_application_type():
+    """Test AnalogInput General Cluster, Application Type Attribute."""
+    app_type = zcl.clusters.general_const.ApplicationType(
+        0x00070100
+    )  # Group 0x00, Type 0x01, Application 0x0007
+
+    assert app_type.group == 0x00
+    assert (
+        app_type.type
+        == zcl.clusters.general_const.AnalogInputType.Relative_Humidity_Percent
+    )
+    assert (
+        app_type.index
+        == zcl.clusters.general_const.RelativeHumidityPercent.Space_Humidity
+    )
