@@ -3,7 +3,9 @@ from __future__ import annotations
 import abc
 import asyncio
 import collections
+from collections.abc import AsyncGenerator, Coroutine
 import contextlib
+from datetime import datetime, timezone
 import errno
 import logging
 import os
@@ -11,7 +13,7 @@ import random
 import sys
 import time
 import typing
-from typing import Any, Coroutine, TypeVar
+from typing import Any, TypeVar
 import warnings
 
 if sys.version_info[:2] < (3, 11):
@@ -22,7 +24,8 @@ else:
 import zigpy.appdb
 import zigpy.backups
 import zigpy.config as conf
-import zigpy.const as const
+from zigpy.const import INTERFERENCE_MESSAGE
+from zigpy.datastructures import PriorityDynamicBoundedSemaphore
 import zigpy.device
 import zigpy.endpoint
 import zigpy.exceptions
@@ -68,16 +71,16 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self._dblistener = None
         self._groups = zigpy.group.Groups(self)
         self._listeners = {}
-        self._ota = zigpy.ota.OTA(self)
         self._send_sequence = 0
         self._tasks: set[asyncio.Future[Any]] = set()
 
         self._watchdog_task: asyncio.Task | None = None
 
-        self._concurrent_requests_semaphore = zigpy.util.DynamicBoundedSemaphore(
+        self._concurrent_requests_semaphore = PriorityDynamicBoundedSemaphore(
             self._config[conf.CONF_MAX_CONCURRENT_REQUESTS]
         )
 
+        self.ota = zigpy.ota.OTA(self._config[conf.CONF_OTA], self)
         self.backups: zigpy.backups.BackupManager = zigpy.backups.BackupManager(self)
         self.topology: zigpy.topology.Topology = zigpy.topology.Topology(self)
 
@@ -185,23 +188,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             # Some radios (like the Conbee) can fail to deliver the startup broadcast
             # due to interference
             LOGGER.warning("Failed to send startup broadcast: %s", e)
-            LOGGER.warning(const.INTERFERENCE_MESSAGE)
-
-        if self.config[conf.CONF_STARTUP_ENERGY_SCAN]:
-            # Each scan period is 15.36ms. Scan for at least 200ms (2^4 + 1 periods) to
-            # pick up WiFi beacon frames.
-            results = await self.energy_scan(
-                channels=t.Channels.ALL_CHANNELS, duration_exp=4, count=1
-            )
-            LOGGER.debug("Startup energy scan: %s", results)
-
-            if results[self.state.network_info.channel] > ENERGY_SCAN_WARN_THRESHOLD:
-                LOGGER.warning(
-                    "Zigbee channel %s utilization is %0.2f%%!",
-                    self.state.network_info.channel,
-                    100 * results[self.state.network_info.channel] / 255,
-                )
-                LOGGER.warning(const.INTERFERENCE_MESSAGE)
+            LOGGER.warning(INTERFERENCE_MESSAGE)
 
         if self.config[conf.CONF_NWK_BACKUP_ENABLED]:
             self.backups.start_periodic_backups(
@@ -215,8 +202,16 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 period=(60 * self.config[zigpy.config.CONF_TOPO_SCAN_PERIOD])
             )
 
-        # Only initialize OTA after we've fully loaded
-        await self.ota.initialize()
+        if (
+            self.config[conf.CONF_OTA][conf.CONF_OTA_ENABLED]
+            and self.config[conf.CONF_OTA][conf.CONF_OTA_BROADCAST_ENABLED]
+        ):
+            self.ota.start_periodic_broadcasts(
+                initial_delay=self._config[conf.CONF_OTA][
+                    conf.CONF_OTA_BROADCAST_INITIAL_DELAY
+                ],
+                interval=self._config[conf.CONF_OTA][conf.CONF_OTA_BROADCAST_INTERVAL],
+            )
 
     async def startup(self, *, auto_form: bool = False) -> None:
         """Starts a network, optionally forming one with random settings if necessary."""
@@ -224,13 +219,13 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         try:
             await self.connect()
             await self.initialize(auto_form=auto_form)
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             await self.shutdown(db=False)
 
             if isinstance(e, ConnectionError) or (
                 isinstance(e, OSError) and e.errno in TRANSIENT_CONNECTION_ERRORS
             ):
-                raise zigpy.exceptions.TransientConnectionError() from e
+                raise zigpy.exceptions.TransientConnectionError from e
 
             raise
 
@@ -422,12 +417,13 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         if self._watchdog_task is not None:
             self._watchdog_task.cancel()
 
+        self.ota.stop_periodic_broadcasts()
         self.backups.stop_periodic_backups()
         self.topology.stop_periodic_scans()
 
         try:
             await self.disconnect()
-        except Exception:
+        except Exception:  # noqa: BLE001
             LOGGER.warning("Failed to disconnect from radio", exc_info=True)
 
         if db and self._dblistener:
@@ -435,7 +431,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
             try:
                 await self._dblistener.shutdown()
-            except Exception:
+            except Exception:  # noqa: BLE001
                 LOGGER.warning("Failed to disconnect from database", exc_info=True)
 
     def add_device(self, ieee: t.EUI64, nwk: t.NWK) -> zigpy.device.Device:
@@ -564,7 +560,12 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
         # Not all stacks send a ZDO command when a device joins so the last_seen should
         # be updated
-        dev.update_last_seen()
+        dev.last_seen = datetime.now(timezone.utc)
+
+        # Cancel all pending requests for the device
+        dev._concurrent_requests_semaphore.cancel_waiting(
+            zigpy.exceptions.DeliveryError("Device has re-joined the network")
+        )
 
         if new_join:
             self.listener_event("device_joined", dev)
@@ -584,8 +585,12 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             dev = self.get_device(ieee=ieee)
         except KeyError:
             return
-        else:
-            self.listener_event("device_left", dev)
+
+        dev._concurrent_requests_semaphore.cancel_waiting(
+            zigpy.exceptions.DeliveryError("Device has left the network")
+        )
+
+        self.listener_event("device_left", dev)
 
     def handle_relays(self, nwk: t.NWK, relays: list[t.NWK]) -> None:
         """Called when a list of relaying devices is received."""
@@ -598,8 +603,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 f"discover_unknown_device_from_relays-nwk={nwk!r}",
             )
         else:
-            # `relays` is a property with a setter that emits an event
-            device.relays = relays
+            device.relays = zigpy.util.filter_relays(relays)
 
     @classmethod
     async def probe(cls, device_config: dict[str, Any]) -> bool | dict[str, Any]:
@@ -616,45 +620,37 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             if new_config not in device_configs:
                 device_configs.append(new_config)
 
-        for device_config in device_configs:
-            app = cls(cls.SCHEMA({conf.CONF_DEVICE: device_config}))
+        for config in device_configs:
+            app = cls({conf.CONF_DEVICE: config})
 
             try:
                 await app.connect()
-            except Exception:
-                LOGGER.debug(
-                    "Failed to probe with config %s", device_config, exc_info=True
-                )
+            except Exception:  # noqa: BLE001
+                LOGGER.debug("Failed to probe with config %s", config, exc_info=True)
             else:
-                return device_config
+                return config
             finally:
                 await app.disconnect()
 
         return False
 
     @abc.abstractmethod
-    async def connect(self):
+    async def connect(self) -> None:
         """Connect to the radio hardware and verify that it is compatible with the library.
         This method should be stateless if the connection attempt fails.
         """
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     async def watchdog_feed(self) -> None:
-        """
-        Reset the firmware watchdog timer.
-        """
+        """Reset the firmware watchdog timer."""
         LOGGER.debug("Feeding watchdog")
         await self._watchdog_feed()
 
     async def _watchdog_feed(self) -> None:
-        """
-        Reset the firmware watchdog timer. Implemented by the radio library.
-        """
+        """Reset the firmware watchdog timer. Implemented by the radio library."""
 
     async def _watchdog_loop(self) -> None:
-        """
-        Watchdog loop to periodically test if the stack is still running.
-        """
+        """Watchdog loop to periodically test if the stack is still running."""
 
         LOGGER.debug("Starting watchdog loop")
 
@@ -663,7 +659,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
             try:
                 await self.watchdog_feed()
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 LOGGER.warning("Watchdog failure", exc_info=e)
 
                 # Treat the watchdog failure as a disconnect
@@ -682,26 +678,26 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
     @abc.abstractmethod
     async def disconnect(self):
         """Disconnects from the radio hardware and shuts down the network."""
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     @abc.abstractmethod
     async def start_network(self):
         """Starts a Zigbee network with settings currently stored in the radio hardware."""
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     @abc.abstractmethod
     async def force_remove(self, dev: zigpy.device.Device):
         """Instructs the radio to remove a device with a lower-level leave command. Not all
         radios implement this.
         """
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     @abc.abstractmethod
     async def add_endpoint(self, descriptor: zdo_types.SimpleDescriptor):
         """Registers a new endpoint on the controlled device. Not all radios will implement
         this.
         """
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     async def register_endpoints(self) -> None:
         """Registers all necessary endpoints.
@@ -745,7 +741,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             await self.add_endpoint(endpoint)
 
     @contextlib.asynccontextmanager
-    async def _limit_concurrency(self):
+    async def _limit_concurrency(self, *, priority: int = t.PacketPriority.NORMAL):
         """Async context manager to limit global coordinator request concurrency."""
 
         start_time = time.monotonic()
@@ -758,7 +754,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 self._concurrent_requests_semaphore.num_waiting,
             )
 
-        async with self._concurrent_requests_semaphore:
+        async with self._concurrent_requests_semaphore(priority=priority):
             if was_locked:
                 LOGGER.debug(
                     "Previously delayed request is now running, delayed by %0.2fs",
@@ -771,7 +767,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
     async def send_packet(self, packet: t.ZigbeePacket) -> None:
         """Send a Zigbee packet using the appropriate addressing mode and provided options."""
 
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     def build_source_route_to(self, dest: zigpy.device.Device) -> list[t.NWK] | None:
         """Compute a source route to the destination device."""
@@ -795,6 +791,8 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         expect_reply: bool = True,
         use_ieee: bool = False,
         extended_timeout: bool = False,
+        ask_for_ack: bool | None = None,
+        priority: int = t.PacketPriority.NORMAL,
     ) -> tuple[zigpy.zcl.foundation.Status, str]:
         """Submit and send data out as an unicast transmission.
         :param device: destination device
@@ -827,7 +825,11 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
         tx_options = t.TransmitOptions.NONE
 
-        if not expect_reply:
+        if ask_for_ack is not None:
+            # Prefer `ask_for_ack` to `expect_reply`
+            if ask_for_ack:
+                tx_options |= t.TransmitOptions.ACK
+        elif not expect_reply:
             tx_options |= t.TransmitOptions.ACK
 
         await self.send_packet(
@@ -843,6 +845,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 extended_timeout=extended_timeout,
                 source_route=source_route,
                 tx_options=tx_options,
+                priority=priority,
             )
         )
 
@@ -859,6 +862,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         *,
         hops: int = 0,
         non_member_radius: int = 3,
+        priority: int = t.PacketPriority.NORMAL,
     ):
         """Submit and send data out as a multicast transmission.
         :param group_id: destination multicast address
@@ -888,6 +892,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 tx_options=t.TransmitOptions.NONE,
                 radius=hops,
                 non_member_radius=non_member_radius,
+                priority=priority,
             )
         )
 
@@ -904,6 +909,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         sequence: t.uint8_t,
         data: bytes,
         broadcast_address: t.BroadcastAddress = t.BroadcastAddress.RX_ON_WHEN_IDLE,
+        priority: int = t.PacketPriority.NORMAL,
     ) -> tuple[zigpy.zcl.foundation.Status, str]:
         """Submit and send data out as an unicast transmission.
         :param profile: Zigbee Profile ID to use for outgoing message
@@ -934,6 +940,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 data=t.SerializableBytes(data),
                 tx_options=t.TransmitOptions.NONE,
                 radius=radius,
+                priority=priority,
             )
         )
 
@@ -1008,7 +1015,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                     f"discover_unknown_device_from_packet-nwk={packet.src.address!r}",
                 )
 
-            return
+            return None
 
         self.listener_event(
             "handle_message",
@@ -1173,7 +1180,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         """Permit joining on NCP.
         Not all radios will require this method.
         """
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     async def permit_with_key(self, node: t.EUI64, code: bytes, time_s: int = 60):
         """Permit a node to join with the provided install code bytes."""
@@ -1194,7 +1201,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self, node: t.EUI64, link_key: t.KeyData, time_s: int = 60
     ) -> None:
         """Permit a node to join with the provided link key."""
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     @abc.abstractmethod
     async def write_network_info(
@@ -1206,7 +1213,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         """Writes network and node state to the radio hardware.
         Any information not supported by the radio should be logged as a warning.
         """
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     @abc.abstractmethod
     async def load_network_info(self, *, load_devices: bool = False) -> None:
@@ -1216,13 +1223,53 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                              a while to load should be skipped. For example, device NWK
                              addresses and link keys.
         """
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
 
     @abc.abstractmethod
     async def reset_network_info(self) -> None:
         """Leaves the current network."""
 
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError  # pragma: no cover
+
+    async def network_scan(
+        self, channels: t.Channels, duration_exp: int
+    ) -> AsyncGenerator[t.NetworkBeacon, None]:
+        """Scans for 802.15.4 networks with a specified duration exponent."""
+        async for network in self._network_scan(
+            channels=channels, duration_exp=duration_exp
+        ):
+            yield network
+
+    # @abc.abstractmethod
+    async def _network_scan(
+        self, channels: t.Channels, duration_exp: int
+    ) -> AsyncGenerator[t.NetworkBeacon, None]:
+        """Scans for 802.15.4 networks with a specified duration exponent."""
+        if False:
+            yield  # pragma: no cover
+
+    async def packet_capture(
+        self, channel: int
+    ) -> AsyncGenerator[t.CapturedPacket, None]:
+        """Packet capture on the specified channel."""
+        async for packet in self._packet_capture(channel=channel):
+            yield packet
+
+    # @abc.abstractmethod
+    async def _packet_capture(
+        self, channel: int
+    ) -> AsyncGenerator[t.CapturedPacket, None]:
+        """Packet capture on the specified channel, internal."""
+        if False:
+            yield  # pragma: no cover
+
+    async def packet_capture_change_channel(self, channel: int) -> None:
+        """Change the channel of an active packet capture."""
+        await self._packet_capture_change_channel(channel=channel)
+
+    # @abc.abstractmethod
+    async def _packet_capture_change_channel(self, channel: int) -> None:
+        """Change the channel of an active packet capture, internal."""
 
     async def permit(self, time_s: int = 60, node: t.EUI64 | str | None = None) -> None:
         """Permit joining on a specific node or all router nodes."""
@@ -1299,28 +1346,15 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         dstaddr.endpoint = self.get_endpoint_id(cluster.cluster_id, cluster.is_server)
         return dstaddr
 
-    def update_config(self, partial_config: dict[str, Any]) -> None:
-        """Update existing config."""
-        self.config = {**self.config, **partial_config}
-
     @property
     def config(self) -> dict:
         """Return current configuration."""
         return self._config
 
-    @config.setter
-    def config(self, new_config) -> None:
-        """Configuration setter."""
-        self._config = self.SCHEMA(new_config)
-
     @property
     def groups(self) -> zigpy.group.Groups:
         return self._groups
 
-    @property
-    def ota(self) -> zigpy.ota.OTA:
-        return self._ota
-
     @property
     def _device(self) -> zigpy.device.Device:
         """The device being controlled."""
