@@ -69,7 +69,7 @@ class NetworkBackup(t.BaseDataclassMixin):
         """Checks if this backup captures enough network state to recreate the network."""
 
         return (
-            self.node_info.ieee != t.EUI64.UNKNOWN
+            self.node_info.ieee != t.EUI64.UNKNOWN  # noqa: PLR1714
             and self.network_info.extended_pan_id != t.EUI64.UNKNOWN
             and self.network_info.pan_id not in (0x0000, 0xFFFF)
             and self.network_info.channel in range(11, 26 + 1)
@@ -206,7 +206,7 @@ class BackupManager(ListenableMixin):
 
         self.backups.append(backup)
 
-    def start_periodic_backups(self, period: int | float) -> None:
+    def start_periodic_backups(self, period: float) -> None:
         self.stop_periodic_backups()
         self._backup_task = asyncio.create_task(self._backup_loop(period))
 
@@ -214,11 +214,11 @@ class BackupManager(ListenableMixin):
         if self._backup_task is not None:
             self._backup_task.cancel()
 
-    async def _backup_loop(self, period: int | float):
+    async def _backup_loop(self, period: float):
         while True:
             try:
                 await self.create_backup()
-            except Exception:
+            except Exception:  # noqa: BLE001
                 LOGGER.warning("Failed to create a network backup", exc_info=True)
 
             LOGGER.debug("Waiting for %ss before backing up again", period)
