@@ -1,10 +1,16 @@
 from __future__ import annotations
 
+import typing
+
 import zigpy.types as t
 
+if typing.TYPE_CHECKING:
+    from zigpy.config import CONF_OTA_PROVIDER_TYPE
+
+CONF_OTA_PROVIDER_TYPE = "type"
+
 CONF_DEVICE_BAUDRATE_DEFAULT = 115200
 CONF_DEVICE_FLOW_CONTROL_DEFAULT = None
-CONF_STARTUP_ENERGY_SCAN_DEFAULT = True
 CONF_MAX_CONCURRENT_REQUESTS_DEFAULT = 8
 CONF_NWK_BACKUP_ENABLED_DEFAULT = True
 CONF_NWK_BACKUP_PERIOD_DEFAULT = 24 * 60  # 24 hours
@@ -18,14 +24,26 @@ CONF_NWK_TC_ADDRESS_DEFAULT = None
 CONF_NWK_TC_LINK_KEY_DEFAULT = t.KeyData(b"ZigBeeAlliance09")
 CONF_NWK_UPDATE_ID_DEFAULT = 0x00
 CONF_NWK_VALIDATE_SETTINGS_DEFAULT = False
-CONF_OTA_ALLOW_FILE_PROVIDERS_DEFAULT = False
-CONF_OTA_IKEA_DEFAULT = False
-CONF_OTA_INOVELLI_DEFAULT = True
-CONF_OTA_LEDVANCE_DEFAULT = True
-CONF_OTA_OTAU_DIR_DEFAULT = None
-CONF_OTA_SALUS_DEFAULT = True
-CONF_OTA_SONOFF_DEFAULT = True
-CONF_OTA_THIRDREALITY_DEFAULT = True
+CONF_OTA_ENABLED_DEFAULT = True
+CONF_OTA_DISABLE_DEFAULT_PROVIDERS_DEFAULT: list[str] = []
+CONF_OTA_BROADCAST_ENABLED_DEFAULT = True
+CONF_OTA_BROADCAST_INITIAL_DELAY_DEFAULT = 3.9 * 60 * 60  # 3.9 hours
+CONF_OTA_BROADCAST_INTERVAL_DEFAULT = 3.9 * 60 * 60  # 3.9 hours
+CONF_OTA_PROVIDERS_DEFAULT = [
+    {
+        CONF_OTA_PROVIDER_TYPE: "ledvance",
+    },
+    {
+        CONF_OTA_PROVIDER_TYPE: "sonoff",
+    },
+    {
+        CONF_OTA_PROVIDER_TYPE: "inovelli",
+    },
+    {
+        CONF_OTA_PROVIDER_TYPE: "thirdreality",
+    },
+]
+CONF_OTA_EXTRA_PROVIDERS_DEFAULT: list[dict[str, typing.Any]] = []
 CONF_SOURCE_ROUTING_DEFAULT = False
 CONF_TOPO_SCAN_PERIOD_DEFAULT = 4 * 60  # 4 hours
 CONF_TOPO_SCAN_ENABLED_DEFAULT = True
