@@ -1,22 +1,28 @@
 from __future__ import annotations
 
 import asyncio
+import contextlib
 from datetime import datetime, timezone
 import enum
 import itertools
 import logging
 import sys
+import time
 import typing
 import warnings
 
-from zigpy.ota.manager import update_firmware
+from zigpy.ota.manager import find_ota_cluster, update_firmware
+from zigpy.zcl.clusters.general import Ota
 
 if sys.version_info[:2] < (3, 11):
     from async_timeout import timeout as asyncio_timeout  # pragma: no cover
 else:
     from asyncio import timeout as asyncio_timeout  # pragma: no cover
 
+from zigpy import zdo
 from zigpy.const import (
+    APS_REPLY_TIMEOUT,
+    APS_REPLY_TIMEOUT_EXTENDED,
     SIG_ENDPOINTS,
     SIG_EP_INPUT,
     SIG_EP_OUTPUT,
@@ -26,24 +32,31 @@ from zigpy.const import (
     SIG_MODEL,
     SIG_NODE_DESC,
 )
+import zigpy.datastructures
 import zigpy.endpoint
 import zigpy.exceptions
 import zigpy.listeners
 import zigpy.types as t
 from zigpy.typing import AddressingMode
 import zigpy.util
-import zigpy.zcl.foundation as foundation
-import zigpy.zdo as zdo
+from zigpy.zcl import foundation
 import zigpy.zdo.types as zdo_t
 
 if typing.TYPE_CHECKING:
     from zigpy.application import ControllerApplication
-    from zigpy.ota.image import BaseOTAImage
+    from zigpy.ota.providers import OtaImageWithMetadata
+
 
-APS_REPLY_TIMEOUT = 5
-APS_REPLY_TIMEOUT_EXTENDED = 28
 LOGGER = logging.getLogger(__name__)
 
+PACKET_DEBOUNCE_WINDOW = 10
+MAX_DEVICE_CONCURRENCY = 1
+
+AFTER_OTA_ATTR_READ_DELAY = 10
+OTA_RETRY_DECORATOR = zigpy.util.retryable_request(
+    tries=4, delay=AFTER_OTA_ATTR_READ_DELAY
+)
+
 
 class Status(enum.IntEnum):
     """The status of a Device. Maintained for backwards compatibility."""
@@ -82,9 +95,37 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         self._skip_configuration: bool = False
         self._send_sequence: int = 0
 
+        self._packet_debouncer = zigpy.datastructures.Debouncer()
+        self._concurrent_requests_semaphore = (
+            zigpy.datastructures.PriorityDynamicBoundedSemaphore(MAX_DEVICE_CONCURRENCY)
+        )
+
         # Retained for backwards compatibility, will be removed in a future release
         self.status = Status.NEW
 
+    @contextlib.asynccontextmanager
+    async def _limit_concurrency(self, *, priority: int = 0):
+        """Async context manager to limit device request concurrency."""
+
+        start_time = time.monotonic()
+        was_locked = self._concurrent_requests_semaphore.locked()
+
+        if was_locked:
+            LOGGER.debug(
+                "Device concurrency (%s) reached, delaying device request (%s enqueued)",
+                self._concurrent_requests_semaphore.max_value,
+                self._concurrent_requests_semaphore.num_waiting,
+            )
+
+        async with self._concurrent_requests_semaphore(priority=priority):
+            if was_locked:
+                LOGGER.debug(
+                    "Previously delayed device request is now running, delayed by %0.2fs",
+                    time.monotonic() - start_time,
+                )
+
+            yield
+
     def get_sequence(self) -> t.uint8_t:
         self._send_sequence = (self._send_sequence + 1) % 256
         return self._send_sequence
@@ -107,7 +148,7 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         return self._last_seen.timestamp() if self._last_seen is not None else None
 
     @last_seen.setter
-    def last_seen(self, value: datetime | int | float):
+    def last_seen(self, value: datetime | float):
         if isinstance(value, (int, float)):
             value = datetime.fromtimestamp(value, timezone.utc)
 
@@ -176,7 +217,10 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
     async def get_node_descriptor(self) -> zdo_t.NodeDescriptor:
         self.info("Requesting 'Node Descriptor'")
 
-        status, _, node_desc = await self.zdo.Node_Desc_req(self.nwk)
+        status, _, node_desc = await self.zdo.Node_Desc_req(
+            self.nwk,
+            priority=t.PacketPriority.HIGH,
+        )
 
         if status != zdo_t.Status.SUCCESS:
             raise zigpy.exceptions.InvalidResponse(
@@ -193,7 +237,7 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
             await self._initialize()
         except (asyncio.TimeoutError, zigpy.exceptions.ZigbeeException):
             self.application.listener_event("device_init_failure", self)
-        except Exception:
+        except Exception:  # noqa: BLE001
             LOGGER.warning(
                 "Device %r failed to initialize due to unexpected error",
                 self,
@@ -219,7 +263,9 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         else:
             self.info("Discovering endpoints")
 
-            status, _, endpoints = await self.zdo.Active_EP_req(self.nwk)
+            status, _, endpoints = await self.zdo.Active_EP_req(
+                self.nwk, priority=t.PacketPriority.HIGH
+            )
 
             if status != zdo_t.Status.SUCCESS:
                 raise zigpy.exceptions.InvalidResponse(
@@ -277,7 +323,7 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         self.endpoints[endpoint_id] = ep
         return ep
 
-    async def add_to_group(self, grp_id: int, name: str = None) -> None:
+    async def add_to_group(self, grp_id: int, name: str | None = None) -> None:
         for ep in self.non_zdo_endpoints:
             await ep.add_to_group(grp_id, name)
 
@@ -296,6 +342,8 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         expect_reply=True,
         timeout=APS_REPLY_TIMEOUT,
         use_ieee=False,
+        ask_for_ack: bool | None = None,
+        priority: int = t.PacketPriority.NORMAL,
     ):
         extended_timeout = False
 
@@ -306,28 +354,31 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
 
         # Use a lambda so we don't leave the coroutine unawaited in case of an exception
         send_request = lambda: self._application.request(  # noqa: E731
-            self,
-            profile,
-            cluster,
-            src_ep,
-            dst_ep,
-            sequence,
-            data,
+            device=self,
+            profile=profile,
+            cluster=cluster,
+            src_ep=src_ep,
+            dst_ep=dst_ep,
+            sequence=sequence,
+            data=data,
             expect_reply=expect_reply,
             use_ieee=use_ieee,
             extended_timeout=extended_timeout,
+            ask_for_ack=ask_for_ack,
+            priority=priority,
         )
 
-        if not expect_reply:
-            await send_request()
-            return None
+        async with self._limit_concurrency(priority=priority):
+            if not expect_reply:
+                await send_request()
+                return None
 
-        # Only create a pending request if we are expecting a reply
-        with self._pending.new(sequence) as req:
-            await send_request()
+            # Only create a pending request if we are expecting a reply
+            with self._pending.new(sequence) as req:
+                await send_request()
 
-            async with asyncio_timeout(timeout):
-                return await req.result
+                async with asyncio_timeout(timeout):
+                    return await req.result
 
     def handle_message(
         self,
@@ -383,6 +434,14 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         if packet.rssi is not None:
             self.rssi = packet.rssi
 
+        if self._packet_debouncer.filter(
+            # Be conservative with deduplication
+            obj=packet.replace(timestamp=None, tsn=None, lqi=None, rssi=None),
+            expire_in=PACKET_DEBOUNCE_WINDOW,
+        ):
+            self.debug("Filtering duplicate packet")
+            return
+
         # Filter out packets that refer to unknown endpoints or clusters
         if packet.src_ep not in self.endpoints:
             self.debug(
@@ -427,7 +486,7 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
                 # Next, parse the ZCL/ZDO payload
                 # FIXME: ZCL deserialization mutates the header!
                 hdr, args = endpoint.deserialize(packet.cluster_id, data)
-        except Exception as exc:
+        except Exception as exc:  # noqa: BLE001
             error = zigpy.exceptions.ParsingError()
             error.__cause__ = exc
 
@@ -483,42 +542,71 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         )
 
     async def reply(
-        self, profile, cluster, src_ep, dst_ep, sequence, data, use_ieee=False
+        self,
+        profile,
+        cluster,
+        src_ep,
+        dst_ep,
+        sequence,
+        data,
+        timeout=APS_REPLY_TIMEOUT,
+        expect_reply: bool = False,
+        use_ieee: bool = False,
+        ask_for_ack: bool | None = None,
+        priority: int = t.PacketPriority.NORMAL,
     ):
         return await self.request(
-            profile,
-            cluster,
-            src_ep,
-            dst_ep,
-            sequence,
-            data,
-            expect_reply=False,
+            profile=profile,
+            cluster=cluster,
+            src_ep=src_ep,
+            dst_ep=dst_ep,
+            sequence=sequence,
+            data=data,
+            expect_reply=expect_reply,
+            timeout=timeout,
             use_ieee=use_ieee,
+            ask_for_ack=ask_for_ack,
+            priority=priority,
         )
 
     async def update_firmware(
         self,
-        firmware_image: BaseOTAImage,
-        progress_callback: callable = None,
+        image: OtaImageWithMetadata,
+        progress_callback: callable | None = None,
         force: bool = False,
     ) -> foundation.Status:
         """Update device firmware."""
         if self.ota_in_progress:
             self.debug("OTA already in progress")
-            return
+            return None
 
         self.ota_in_progress = True
 
         try:
             result = await update_firmware(
-                self, firmware_image, progress_callback, force
+                device=self,
+                image=image,
+                progress_callback=progress_callback,
+                force=force,
             )
-        except Exception as exc:
-            self.ota_in_progress = False
+        except Exception as exc:  # noqa: BLE001
             self.debug("OTA failed!", exc_info=exc)
-            raise exc
+            raise
+        finally:
+            self.ota_in_progress = False
+
+        if result != foundation.Status.SUCCESS:
+            return result
+
+        # Clear the current file version when the update succeeds
+        ota = find_ota_cluster(self)
+        ota.update_attribute(Ota.AttributeDefs.current_file_version.id, None)
+
+        await asyncio.sleep(AFTER_OTA_ATTR_READ_DELAY)
+        await OTA_RETRY_DECORATOR(ota.read_attributes)(
+            [Ota.AttributeDefs.current_file_version.name]
+        )
 
-        self.ota_in_progress = False
         return result
 
     def radio_details(self, lqi=None, rssi=None) -> None:
@@ -529,7 +617,7 @@ class Device(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
 
     def log(self, lvl, msg, *args, **kwargs) -> None:
         msg = "[0x%04x] " + msg
-        args = (self.nwk,) + args
+        args = (self.nwk, *args)
         LOGGER.log(lvl, msg, *args, **kwargs)
 
     @property
