@@ -5,6 +5,8 @@ from datetime import datetime, timezone
 import enum
 import typing
 
+import attrs
+
 from . import basic
 from .struct import Struct
 
@@ -13,8 +15,12 @@ if typing.TYPE_CHECKING:
 
 
 class BaseDataclassMixin:
-    def replace(self, **kwargs) -> Self:
-        return dataclasses.replace(self, **kwargs)
+    def replace(self, **kwargs: typing.Any) -> Self:
+        if dataclasses.is_dataclass(self):
+            assert not isinstance(self, type)  # `is_dataclass` works on types as well
+            return dataclasses.replace(self, **kwargs)
+        else:
+            return attrs.evolve(self, **kwargs)
 
 
 def _hex_string_to_bytes(hex_string: str) -> bytes:
@@ -39,9 +45,9 @@ class BroadcastAddress(basic.enum16):
 class EUI64(basic.FixedList, item_type=basic.uint8_t, length=8):
     # EUI 64-bit ID (an IEEE address).
     def __repr__(self) -> str:
-        return ":".join("%02x" % i for i in self[::-1])
+        return ":".join(f"{i:02x}" for i in self[::-1])
 
-    def __hash__(self) -> int:  # type: ignore
+    def __hash__(self) -> int:  # type: ignore[override]
         return hash(repr(self))
 
     @classmethod
@@ -551,6 +557,9 @@ class AddrModeAddress(BaseDataclassMixin):
                 AddrMode.Broadcast: BroadcastAddress,
             }[self.addr_mode](self.address)
 
+    def __hash__(self) -> int:
+        return hash((self.addr_mode, self.address))
+
 
 class TransmitOptions(enum.Flag):
     NONE = 0
@@ -559,6 +568,15 @@ class TransmitOptions(enum.Flag):
     APS_Encryption = 2
 
 
+class PacketPriority(enum.IntEnum):
+    """Packet priority"""
+
+    CRITICAL = 2
+    HIGH = 1
+    NORMAL = 0
+    LOW = -1
+
+
 @dataclasses.dataclass
 class ZigbeePacket(BaseDataclassMixin):
     """Container for the information in an incoming or outgoing ZDO or ZCL packet.
@@ -571,6 +589,9 @@ class ZigbeePacket(BaseDataclassMixin):
         compare=False, default_factory=lambda: datetime.now(timezone.utc)
     )
 
+    # Higher priority will try to be sent before lower
+    priority: int = dataclasses.field(default=0)
+
     # Set to `None` when the packet is outgoing
     src: AddrModeAddress | None = dataclasses.field(default=None)
     src_ep: basic.uint8_t | None = dataclasses.field(default=None)
@@ -600,3 +621,66 @@ class ZigbeePacket(BaseDataclassMixin):
     # Options for incoming packets
     lqi: basic.uint8_t | None = dataclasses.field(default=None)
     rssi: basic.int8s | None = dataclasses.field(default=None)
+
+    def __hash__(self) -> int:
+        return hash(
+            (
+                self.timestamp,
+                self.src,
+                self.src_ep,
+                self.dst,
+                self.dst_ep,
+                self.source_route,
+                self.extended_timeout,
+                self.tsn,
+                self.profile_id,
+                self.cluster_id,
+                self.data,
+                self.tx_options,
+                self.radius,
+                self.non_member_radius,
+                self.lqi,
+                self.rssi,
+                self.priority,
+            )
+        )
+
+
+@dataclasses.dataclass(frozen=True)
+class NetworkBeacon(BaseDataclassMixin):
+    pan_id: PanId
+    extended_pan_id: EUI64
+    channel: basic.uint8_t
+    permit_joining: bool
+    stack_profile: basic.uint8_t
+    nwk_update_id: basic.uint8_t
+    lqi: basic.uint8_t
+
+    # Migrate to kwarg-only once we drop 3.9
+    src: NWK | None = None
+    rssi: basic.int8s | None = None
+    depth: basic.uint8_t | None = None
+    router_capacity: bool | None = None
+    device_capacity: bool | None = None
+    protocol_version: basic.uint8_t | None = None
+
+
+@dataclasses.dataclass(frozen=True)
+class CapturedPacket(BaseDataclassMixin):
+    timestamp: datetime
+    rssi: float
+    lqi: basic.uint8_t
+    channel: basic.uint8_t
+    data: bytes
+
+    def compute_fcs(self) -> bytes:
+        crc = 0x0000
+
+        for c in self.data:
+            q = (crc ^ c) & 15  # Do low-order 4 bits
+            crc = (crc // 16) ^ (q * 0x1081)
+
+            q = (crc ^ (c // 16)) & 15  # And high 4 bits
+            crc = (crc // 16) ^ (q * 0x1081)
+
+        return crc.to_bytes(2, "little")
