@@ -13,7 +13,7 @@ from zigpy.zcl.foundation import (
 
 
 class ApplianceIdentification(Cluster):
-    cluster_id: Final = 0x0B00
+    cluster_id: Final[t.uint16_t] = 0x0B00
     name: Final = "Appliance Identification"
     ep_attribute: Final = "appliance_id"
 
@@ -51,7 +51,7 @@ class ApplianceIdentification(Cluster):
 
 
 class MeterIdentification(Cluster):
-    cluster_id: Final = 0x0B01
+    cluster_id: Final[t.uint16_t] = 0x0B01
     name: Final = "Meter Identification"
     ep_attribute: Final = "meter_id"
 
@@ -95,7 +95,7 @@ class MeterIdentification(Cluster):
 
 
 class ApplianceEventAlerts(Cluster):
-    cluster_id: Final = 0x0B02
+    cluster_id: Final[t.uint16_t] = 0x0B02
     name: Final = "Appliance Event Alerts"
     ep_attribute: Final = "appliance_event"
 
@@ -113,7 +113,7 @@ class ApplianceEventAlerts(Cluster):
 
 
 class ApplianceStatistics(Cluster):
-    cluster_id: Final = 0x0B03
+    cluster_id: Final[t.uint16_t] = 0x0B03
     name: Final = "Appliance Statistics"
     ep_attribute: Final = "appliance_stats"
 
@@ -139,37 +139,37 @@ class ApplianceStatistics(Cluster):
 
 
 class MeasurementType(t.bitmap32):
-    Active_measurement_AC = 2 << 0
-    Reactive_measurement_AC = 2 << 1
-    Apparent_measurement_AC = 2 << 2
-    Phase_A_measurement = 2 << 3
-    Phase_B_measurement = 2 << 4
-    Phase_C_measurement = 2 << 5
-    DC_measurement = 2 << 6
-    Harmonics_measurement = 2 << 7
-    Power_quality_measurement = 2 << 8
+    Active_measurement_AC = 1 << 0
+    Reactive_measurement_AC = 1 << 1
+    Apparent_measurement_AC = 1 << 2
+    Phase_A_measurement = 1 << 3
+    Phase_B_measurement = 1 << 4
+    Phase_C_measurement = 1 << 5
+    DC_measurement = 1 << 6
+    Harmonics_measurement = 1 << 7
+    Power_quality_measurement = 1 << 8
 
 
 class DCOverloadAlarmMark(t.bitmap8):
-    Voltage_Overload = 0b00000001
-    Current_Overload = 0b00000010
+    Voltage_Overload = 1 << 0
+    Current_Overload = 1 << 1
 
 
 class ACAlarmsMask(t.bitmap16):
-    Voltage_Overload = 2 << 0
-    Current_Overload = 2 << 1
-    Active_Power_Overload = 2 << 2
-    Reactive_Power_Overload = 2 << 3
-    Average_RMS_Over_Voltage = 2 << 4
-    Average_RMS_Under_Voltage = 2 << 5
-    RMS_Extreme_Over_Voltage = 2 << 6
-    RMS_Extreme_Under_Voltage = 2 << 7
-    RMS_Voltage_Sag = 2 << 8
-    RMS_Voltage_Swell = 2 << 9
+    Voltage_Overload = 1 << 0
+    Current_Overload = 1 << 1
+    Active_Power_Overload = 1 << 2
+    Reactive_Power_Overload = 1 << 3
+    Average_RMS_Over_Voltage = 1 << 4
+    Average_RMS_Under_Voltage = 1 << 5
+    RMS_Extreme_Over_Voltage = 1 << 6
+    RMS_Extreme_Under_Voltage = 1 << 7
+    RMS_Voltage_Sag = 1 << 8
+    RMS_Voltage_Swell = 1 << 9
 
 
 class ElectricalMeasurement(Cluster):
-    cluster_id: Final = 0x0B04
+    cluster_id: Final[t.uint16_t] = 0x0B04
     name: Final = "Electrical Measurement"
     ep_attribute: Final = "electrical_measurement"
 
@@ -543,7 +543,7 @@ class ElectricalMeasurement(Cluster):
 
 
 class Diagnostic(Cluster):
-    cluster_id: Final = 0x0B05
+    cluster_id: Final[t.uint16_t] = 0x0B05
     ep_attribute: Final = "diagnostic"
 
     class AttributeDefs(BaseAttributeDefs):
